program CalcSend;

uses
	Windows,
	Messages,
	SysUtils,
	Classes,
	CommCtrl,
	LinkImportUnit in 'LinkImportUnit.pas';

{$I StringInc}

const
	CHILD_ANI = 500;
	CHILD_FILE = 501;
	CHILD_PATH = 502;
	CHILD_PROGBAR = 503;
	CHILD_CANCELBUTTON = 504;

var
	Closed: Boolean = False;
	Wnd: THandle;

function WndProc(Wnd: HWnd; Msg: UInt; wParam: WParam; lParam: LParam): LResult; stdcall;
begin
	if (Msg = WM_CLOSE)
		or ((Msg = WM_COMMAND) and (Word (wParam) = CHILD_CANCELBUTTON)) then begin
		Closed := True;
		PostMessage (Wnd, WM_QUIT, 1, 0);
		Result := 0;
	end else
		Result := DefWindowProc (Wnd, Msg, wParam, lParam);
end;

function ProgressCallBack(ID: Pointer; Progress: PDWord): TCalcBool; stdcall;
var
	Msg: TMsg;
begin
	Result := True;
	SendMessage (HWnd (ID), PBM_SETPOS, Progress^ div 10, 0);
	while PeekMessage (Msg, Wnd, 0, 0, 0) do
		if GetMessage (Msg, Wnd, 0, 0) and
			(not ((Msg.Message = WM_CHAR) and ((Msg.wParam = VK_RETURN) or (Msg.wParam = VK_ESCAPE)))) then begin
			TranslateMessage (Msg);
			DispatchMessage (Msg);
		end else begin
			Closed := True;
			Result := False;
			Break;
		end;
end;

function DirExists(var Directory: string): Boolean;
var
	S: TSearchRec;
begin
	if Length (Directory) > 0 then begin
		if Directory [Length (Directory)] = '\' then
			Delete (Directory, Length (Directory), 1);
		Result := (FindFirst (Directory, faDirectory, S) = 0);
		FindClose (S);
	end else
		Result := False;
end;

var
	Files: TStringList;
	CurFolderStr: string;

procedure AddToFiles(Name: string);
var
	R: Integer;
	S: TSearchRec;
begin
	if FileExists (Name) then
		Files.Add (Name + '=' + CurFolderStr)
	else if DirExists (Name) then begin
		R := FindFirst (Name + '\*.*', faAnyFile or faDirectory, S);
		while R = 0 do begin
			if (S.Name <> '.') and (S.Name <> '..') then
				AddToFiles (Name + '\' + S.Name);
			R := FindNext (S);
		end;
		FindClose (S);
	end;
end;

var
	Link: TLinkHandle;
	I,
	ParamIdx: Integer;
	Param,
	FileName,
	FilePath: string;
	CurRealFolder: array [0..9] of Char;
	CompleteName: array [0..MAX_PATH+9] of Char;
	Size: Word;
	Progress: DWord;
	Total: Integer;
	WndClass: TWndClass;
	Ani,
	FileLabel,
	PathLabel,
	ProgBar,
	CancelButton: HWnd;
	Font: HFont;
	ShellLib: HModule;

begin
	if InitializeLink and TryOpenConnection (Link, 0) then begin
		ShellLib := LoadLibrary ('SHELL32.DLL');
		InitCommonControls;
		FillChar (WndClass, SizeOf (WndClass), 0);
		WndClass.lpszClassName := 'CalcCopyWnd';
		WndClass.lpfnWndProc := @WndProc;
		WndClass.hInstance := ShellLib;
		WndClass.hCursor := LoadCursor (0, IDC_ARROW);
		WndClass.hbrBackground := COLOR_BTNFACE + 1;
		Windows.RegisterClass (WndClass);
		Font := CreateFont (8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 'MS Sans Serif');
		Wnd := CreateWindowEx (WS_EX_DLGMODALFRAME or WS_EX_WINDOWEDGE, 'CalcCopyWnd', PChar (AnsiString (GetUIString (UICopying))), WS_POPUP or WS_CAPTION or WS_CLIPCHILDREN or WS_SYSMENU or DS_3DLOOK or DS_NOIDLEMSG, 50, 50, 410, 160, 0, 0, ShellLib, nil);
		Ani := CreateWindow (ANIMATE_CLASS, nil, WS_CHILD or WS_VISIBLE or ACS_TRANSPARENT, 16, 0, 0, 0, Wnd, CHILD_ANI, ShellLib, nil);
		Animate_Open (Ani, PChar (Word (161)));
		FileLabel := CreateWindow ('STATIC', nil, WS_CHILD or WS_VISIBLE, 16, 66, 264, 14, Wnd, CHILD_FILE, ShellLib, nil);
		PathLabel := CreateWindow ('STATIC', nil, WS_CHILD or WS_VISIBLE, 16, 82, 264, 14, Wnd, CHILD_PATH, ShellLib, nil);
		ProgBar := CreateWindow (PROGRESS_CLASS, nil, WS_CHILD or WS_VISIBLE, 16, 104, 264, 15, Wnd, CHILD_PROGBAR, ShellLib, nil);
		CancelButton := CreateWindow ('BUTTON', PChar (AnsiString (GetUIString (UICancel))), WS_CHILD or WS_VISIBLE or BS_DEFPUSHBUTTON or BS_TEXT or BS_CENTER or BS_VCENTER, 296, 94, 92, 25, Wnd, CHILD_CANCELBUTTON, ShellLib, nil);
		SendMessage (FileLabel, WM_SETFONT, Font, 0);
		SendMessage (PathLabel, WM_SETFONT, Font, 0);
		SendMessage (CancelButton, WM_SETFONT, Font, 0);
		SetFocus (CancelButton);
		ShowWindow (Wnd, SW_SHOW);
		UpdateWindow (Wnd);
		Animate_Play (Ani, 0, Word (-1), Word (-1));
		Files := TStringList.Create;
		for ParamIdx := 0 to ParamCount do begin
			if (ParamIdx > 0) or (GetCommandLine [0] = '/') then begin
				if ParamIdx > 0 then
					Param := ParamStr (ParamIdx) + ';'
				else begin
					Param := AnsiString (GetCommandLine);
					Delete (Param, Pos (' ', Param), Length (Param));
					Insert (';', Param, Length (Param) + 1);
				end;
				while Length (Param) > 0 do begin
					FileName := Copy (Param, 1, Pos (';', Param) - 1);
					if Length (FileName) > 0 then begin
						if FileName [1] in ['/', '-'] then begin
							if UpCase (FileName [2]) = 'F' then
								CurFolderStr := Trim (Copy (FileName, 4, Length (FileName)));
						end else
							AddToFiles (FileName);
					end;
					Delete (Param, 1, Pos (';', Param));
				end;
			end;
		end;
		if Files.Count > 0 then begin
			Total := 0;
			for ParamIdx := 0 to Files.Count - 1 do
				if CheckFileFormat (Link, PChar (Files.Names [ParamIdx]), nil, nil, @Size) then
					Inc (Total, Size);
			SendMessage (ProgBar, PBM_SETRANGE, 0, MakeLong (0, Word (Total div 10)));
			Progress := 0;
			for ParamIdx := 0 to Files.Count - 1 do begin
				if Closed then
					Break;
				FileName := Files.Strings [ParamIdx];
				CurFolderStr := Copy (FileName, Pos ('=', FileName) + 1, Length (FileName));
				Delete (FileName, Pos ('=', FileName), Length (FileName));
				StrPLCopy (CompleteName, ExtractFileName (FileName), MAX_PATH);
				SetWindowText (FileLabel, CompleteName);
				SetWindowText (PathLabel, '');
				if FileExists (FileName) then begin
					I := StrLen (CompleteName);
					CompleteName [I] := ' ';
					CompleteName [I + 1] := '(';
					if CheckFileFormat (Link, PChar (FileName), CurRealFolder, @(CompleteName[I+2]), nil) then begin
						if Length (CurFolderStr) > 0 then
							StrPLCopy (CurRealFolder, CurFolderStr, 9);
						I := StrLen (CompleteName);
						CompleteName [I] := ')';
						CompleteName [I + 1] := #0;
						SetWindowText (FileLabel, CompleteName);
						FilePath := ExtractFilePath (FileName);
						if Length (FilePath) > 3 then
							FilePath := ExtractFileName (Copy (FilePath, 1, Length (FilePath) - 1));
						SetWindowText (PathLabel, PChar (Format (GetUIString (UICopyPath), [FilePath, AnsiString (CurRealFolder)])));
						while not SendFile (Link, CurRealFolder, nil, PChar (FileName), @Progress, ProgressCallBack, Pointer (ProgBar)) do
							if Closed then
								Break
							else
								case MessageBox (0, PChar (AnsiString (GetUIString (UISendError))), PChar (AnsiString (GetUIString (UIError))), MB_ABORTRETRYIGNORE or MB_DEFBUTTON2 or MB_ICONERROR or MB_TASKMODAL) of
									IDABORT: begin
										Closed := True;
										Break;
									end;
									IDIGNORE:
										Break;
								end;
					end else
						if MessageBox (0, PChar (AnsiString (GetUIString (UIFormatError))), PChar (AnsiString (GetUIString (UIError))), MB_OKCANCEL or MB_ICONERROR or MB_TASKMODAL) = IDCANCEL then
							Closed := True;
				end else
					MessageBox (0, PChar (AnsiString (GetUIString (UIFileExistsError))), PChar (AnsiString (GetUIString (UIError))), MB_OK or MB_ICONERROR or MB_TASKMODAL);
			end;
		end else
			MessageBox (0, GetCommandLine, 'Invalid Command Line', MB_OK or MB_ICONERROR);
		Files.Free;
		Animate_Close (Ani);
		DestroyWindow (CancelButton);
		DestroyWindow (ProgBar);
		DestroyWindow (PathLabel);
		DestroyWindow (FileLabel);
		DestroyWindow (Ani);
		DestroyWindow (Wnd);
		DeleteObject (Font);
		FreeLibrary (ShellLib);
		CloseConnection (Link);
	end;
end.
