unit DropTargetUnit;

interface

uses
	LinkUnit,
	SubFolderUnit,
	Windows, SysUtils, ActiveX, ComObj, ShlObj;

type
	TDropTarget = class(TComObject, IDropTarget)
	private
		FName: TSubFolderName;
	protected
		function DragEnter(const dataObj: IDataObject; grfKeyState: Longint;
			pt: TPoint; var dwEffect: Longint): HResult; stdcall;
		function DragOver(grfKeyState: Longint; pt: TPoint;
			var dwEffect: Longint): HResult; stdcall;
		function DragLeave: HResult; stdcall;
		function Drop(const dataObj: IDataObject; grfKeyState: Longint; pt: TPoint;
			var dwEffect: Longint): HResult; stdcall;
	public
		constructor Create(const Name: TSubFolderName);
		destructor Destroy; override;
	end;

const
	Class_DropTarget: TGUID = '{595D2826-10B5-11D5-A253-00900001489D}';

implementation

uses
	ComServ;

{ TDropTarget }

constructor TDropTarget.Create(const Name: TSubFolderName);
begin
	inherited Create;
	Inc (DLLRefCount);
	FName := Name;
end;

destructor TDropTarget.Destroy;
begin
	Dec (DLLRefCount);
	inherited;
end;

function TDropTarget.DragEnter(const dataObj: IDataObject;
	grfKeyState: Integer; pt: TPoint; var dwEffect: Integer): HResult;
begin
	dwEffect := DROPEFFECT_COPY;
	Result := S_OK;
end;

function TDropTarget.DragLeave: HResult;
begin
	Result := S_OK;
end;

function TDropTarget.DragOver(grfKeyState: Integer; pt: TPoint;
	var dwEffect: Integer): HResult;
begin
	dwEffect := DROPEFFECT_COPY;
	Result := S_OK;
end;

function TDropTarget.Drop(const dataObj: IDataObject; grfKeyState: Integer;
	pt: TPoint; var dwEffect: Integer): HResult;
var
	Format: TFormatEtc;
	FMedium: TStgMedium;
	I: Integer;
	L: PDropFiles;
	P,
	F: PChar;
	S: array [0..9] of Char;
	FileName: array [0..MAX_PATH] of Char;
	WideChars: Boolean;
	StartupInfo: TStartupInfo;
	ProcessInfo: TProcessInformation;
begin
	FillChar (FMedium, SizeOf (FMedium), 0);
	StrPLCopy (S, FName, 9);
	FillChar (Format, SizeOf (Format), 0);
	with Format do begin
		cfFormat := CF_HDROP;
		ptd := nil;
		dwAspect := DVASPECT_CONTENT;
		lIndex := -1;
		TyMed := TYMED_HGLOBAL;
	end;
	Result := DataObj.GetData (Format, FMedium);
	if Result = S_OK then begin
		L := GlobalLock (FMedium.hGlobal);
		WideChars := L.fWide;
		P := PChar (Cardinal (L) + L.pFiles);
		if WideChars then begin
			I := 0;
			repeat
				Inc (I);
				P [I] := P [I * 2];
			until (P [I] = #0) and (P [I - 1] = #0);
		end;
		P := @(P[-1]);
		I := StrLen (S);
		P[-4-I] := '/';
		P[-3-I] := 'f';
		P[-2-I] := ':';
		StrCopy (@(P[-1-I]), S);
		P [-1] := ' ';
		P [0] := '"';
		F := @(P[1]);
		while F [0] <> #0 do begin
			I := StrLen (F);
			if F [I + 1] = #0 then
				F [I] := '"'
			else
				F [I] := ';';
			F := @(F[I+1]);
		end;
		FillChar (StartupInfo, SizeOf (StartupInfo), 0);
		StartupInfo.cb := SizeOf (StartupInfo);
		GetModuleFileName (hInstance, FileName, MAX_PATH + 1);
		F := @(FileName[StrLen(FileName)-1]);
		while (F <> @FileName) and (F [-1] <> '\') do
			F := @(F[-1]);
		StrCopy (F, 'CalcSend.exe');
		if CreateProcess (FileName, @(P[-4-StrLen(S)]), nil, nil, False, CREATE_DEFAULT_ERROR_MODE or NORMAL_PRIORITY_CLASS, nil, nil, StartupInfo, ProcessInfo) then begin
			CloseHandle (ProcessInfo.hThread);
			CloseHandle (ProcessInfo.hProcess);
		end;
		GlobalUnlock (FMedium.hGlobal);
		ReleaseStgMedium (FMedium);
	end;
end;

initialization
	TComObjectFactory.Create(ComServer, TDropTarget, Class_DropTarget,
		'DropTarget', '', ciInternal, tmApartment);
end.
