unit LinkImportUnit;

interface

{$I LinkIntfInc}

const
	FolderLink = 'FolderLink.dll';

function InitializeLink: Boolean;

{ Imported Functions }
var
	{	Library Interface }
	CreateConnection: function(PortID: TLinkPort = CALC_PORT_USER; CableType: TLinkType = CALC_CABLE_USER): TLinkHandle; stdcall;
	OpenConnection: function(Handle: TLinkHandle): TCalcBool; stdcall;
	CloseConnection: procedure(Handle: TLinkHandle); stdcall;

	CalcReady: function(Handle: TLinkHandle): TCalcBool; stdcall;

	SendAck: function(Handle: TLinkHandle): TCalcBool; stdcall;
	SendWait: function(Handle: TLinkHandle): TCalcBool; stdcall;
	SendByte: function(Handle: TLinkHandle; Value: Byte; CheckSum: PWord = nil): TCalcBool; stdcall;
	SendWord: function(Handle: TLinkHandle; Value: Word; CheckSum: PWord = nil): TCalcBool; stdcall;
	SendLongWord: function(Handle: TLinkHandle; Value: LongWord; CheckSum: PWord = nil): TCalcBool; stdcall;
	SendString: function(Handle: TLinkHandle; Value: PChar; CheckSum: PWord = nil): TCalcBool; stdcall;
	SendKeyInternal: function(Handle: TLinkHandle; Key: Word): TCalcBool; stdcall;
	SendKeys: function(Handle: TLinkHandle; Keys: PChar): TCalcBool; stdcall;
	SendVar: function(Handle: TLinkHandle; Folder, Name: PChar; Buffer: Pointer; BufSize: Cardinal; Progress: PDWord = nil; ProgressCallBack: TProgressCallBack = nil; ID: Pointer = nil): TCalcBool; stdcall;
	SendFile: function(Handle: TLinkHandle; Folder, Name: PChar; FileName: PChar; Progress: PDWord = nil; ProgressCallBack: TProgressCallBack = nil; ID: Pointer = nil): TCalcBool; stdcall;
	CheckFileFormat: function(Handle: TLinkHandle; FileName: PChar; Folder: PChar = nil; Name: PChar = nil; Size: PWord = nil): TCalcBool; stdcall;
	ExecuteHomeLine: function(Handle: TLinkHandle; Line: PChar): TCalcBool; stdcall;

	WaitForAck: function(Handle: TLinkHandle; LastResult: PByte = nil): TCalcBool; stdcall;
	CalcWaiting: function(Handle: TLinkHandle; LastResult: PByte = nil): TCalcBool; stdcall;
	ReceiveByte: function(Handle: TLinkHandle; out Value: Byte; CheckSum: PWord = nil): TCalcBool; stdcall;
	ReceiveWord: function(Handle: TLinkHandle; out Value: Word; CheckSum: PWord = nil): TCalcBool; stdcall;
	ReceiveLongWord: function(Handle: TLinkHandle; out Value: LongWord; CheckSum: PWord = nil): TCalcBool; stdcall;

	GetCalcType: function(Handle: TLinkHandle): TCalcType; stdcall;
	GetFolderList: function(Handle: TLinkHandle; Buffer: PChar; Len: Integer): TCalcBool; stdcall;
	GetVarList: function(Handle: TLinkHandle; Folder: PChar; VarListCallBack: TVarListCallBack; ID: Pointer): TCalcBool;

	{ Shell Folder Internal Functions }
	TryOpenConnection: function(out Handle: TLinkHandle; Window: HWnd): Boolean; stdcall;
	GetUIString: function(StringIdx: Integer): ShortString; stdcall;

function SendKey(Handle: TLinkHandle; Key: Char): TCalcBool; stdcall; overload;
function SendKey(Handle: TLinkHandle; Key: Word): TCalcBool; stdcall; overload;

implementation

var
	LibHandle: HModule = 0;

function InitializeLink: Boolean;
var
	PathName: array [0..MAX_PATH] of Char;
	PathLen: DWord;
	OFStruct: TOFStruct;
	RegKey: HKey;
begin
	if LibHandle = 0 then begin
		if OpenFile (FolderLink, OFStruct, OF_EXIST) = HFILE_ERROR then begin
			if RegOpenKeyEx (HKEY_CLASSES_ROOT, 'CLSID\{595D2820-10B5-11D5-A253-00900001489D}\InprocServer32', 0, KEY_QUERY_VALUE, RegKey) = ERROR_SUCCESS then begin
				PathLen := MAX_PATH;
				Result := RegQueryValueEx (RegKey, nil, nil, nil, PByte (@PathName), @PathLen) = ERROR_SUCCESS;
				RegCloseKey (RegKey);
				if not Result then
					Exit;
			end;
		end else
			PathName := FolderLink;
		LibHandle := LoadLibrary (PathName);
		Result := LibHandle <> 0;
		if Result then begin
			CreateConnection := GetProcAddress (LibHandle, 'CreateConnection');
			OpenConnection := GetProcAddress (LibHandle, 'OpenConnection');
			CloseConnection := GetProcAddress (LibHandle, 'CloseConnection');
			CalcReady := GetProcAddress (LibHandle, 'CalcReady');
			SendAck := GetProcAddress (LibHandle, 'SendAck');
			SendWait := GetProcAddress (LibHandle, 'SendWait');
			SendByte := GetProcAddress (LibHandle, 'SendByte');
			SendWord := GetProcAddress (LibHandle, 'SendWord');
			SendLongWord := GetProcAddress (LibHandle, 'SendLongWord');
			SendString := GetProcAddress (LibHandle, 'SendString');
			SendKeyInternal := GetProcAddress (LibHandle, 'SendKey');
			SendKeys := GetProcAddress (LibHandle, 'SendKeys');
			SendVar := GetProcAddress (LibHandle, 'SendVar');
			SendFile := GetProcAddress (LibHandle, 'SendFile');
			CheckFileFormat := GetProcAddress (LibHandle, 'CheckFileFormat');
			ExecuteHomeLine := GetProcAddress (LibHandle, 'ExecuteHomeLine');
			WaitForAck := GetProcAddress (LibHandle, 'WaitForAck');
			CalcWaiting := GetProcAddress (LibHandle, 'CalcWaiting');
			ReceiveByte := GetProcAddress (LibHandle, 'ReceiveByte');
			ReceiveWord := GetProcAddress (LibHandle, 'ReceiveWord');
			ReceiveLongWord := GetProcAddress (LibHandle, 'ReceiveLongWord');
			GetCalcType := GetProcAddress (LibHandle, 'GetCalcType');
			GetFolderList := GetProcAddress (LibHandle, 'GetFolderList');
			GetVarList := GetProcAddress (LibHandle, 'GetVarList');
			TryOpenConnection := GetProcAddress (LibHandle, 'TryOpenConnection');
			GetUIString := GetProcAddress (LibHandle, 'GetUIString');
		end;
	end else
		Result := True;
end;

function SendKey(Handle: TLinkHandle; Key: Char): TCalcBool; stdcall;
begin
	Result := SendKeyInternal (Handle, Byte (Key));
end;

function SendKey(Handle: TLinkHandle; Key: Word): TCalcBool; stdcall;
begin
	Result := SendKeyInternal (Handle, Key);
end;

initialization
finalization
	if LibHandle <> 0 then
		FreeLibrary (LibHandle);
end.
