unit LinkUnit;

{	Uncomment to enable logging of all bytes sent and received to
	C:\Linklog.txt. }
(* {$DEFINE LOGLINK} *)

interface

{$I LinkIntfInc}

{ Exported Functions }
function CreateConnection(PortID: TLinkPort = CALC_PORT_USER; CableType: TLinkType = CALC_CABLE_USER): TLinkHandle; stdcall;
function OpenConnection(Handle: TLinkHandle): TCalcBool; stdcall;
procedure CloseConnection(Handle: TLinkHandle); stdcall;

function CalcReady(Handle: TLinkHandle): TCalcBool; stdcall;

function SendAck(Handle: TLinkHandle): TCalcBool; stdcall;
function SendWait(Handle: TLinkHandle): TCalcBool; stdcall;
function SendByte(Handle: TLinkHandle; Value: Byte; CheckSum: PWord = nil): TCalcBool; stdcall;
function SendWord(Handle: TLinkHandle; Value: Word; CheckSum: PWord = nil): TCalcBool; stdcall;
function SendLongWord(Handle: TLinkHandle; Value: LongWord; CheckSum: PWord = nil): TCalcBool; stdcall;
function SendString(Handle: TLinkHandle; Value: PChar; CheckSum: PWord = nil): TCalcBool; stdcall;
function SendKey(Handle: TLinkHandle; Key: Char): TCalcBool; stdcall; overload;
function SendKey(Handle: TLinkHandle; Key: Word): TCalcBool; stdcall; overload;
function SendKeys(Handle: TLinkHandle; Keys: PChar): TCalcBool; stdcall;
function SendVar(Handle: TLinkHandle; Folder, Name: PChar; Buffer: Pointer; BufSize: Cardinal; Progress: PDWord = nil; ProgressCallBack: TProgressCallBack = nil; ID: Pointer = nil): TCalcBool; stdcall;
function SendFile(Handle: TLinkHandle; Folder, Name: PChar; FileName: PChar; Progress: PDWord = nil; ProgressCallBack: TProgressCallBack = nil; ID: Pointer = nil): TCalcBool; stdcall;
function CheckFileFormat(Handle: TLinkHandle; FileName: PChar; Folder: PChar = nil; Name: PChar = nil; Size: PWord = nil): TCalcBool; stdcall;
function ExecuteHomeLine(Handle: TLinkHandle; Line: PChar): TCalcBool; stdcall;

function WaitForAck(Handle: TLinkHandle; LastResult: PByte = nil): TCalcBool; stdcall;
function CalcWaiting(Handle: TLinkHandle; LastResult: PByte = nil): TCalcBool; stdcall;
function ReceiveByte(Handle: TLinkHandle; out Value: Byte; CheckSum: PWord = nil): TCalcBool; stdcall;
function ReceiveWord(Handle: TLinkHandle; out Value: Word; CheckSum: PWord = nil): TCalcBool; stdcall;
function ReceiveLongWord(Handle: TLinkHandle; out Value: LongWord; CheckSum: PWord = nil): TCalcBool; stdcall;

function GetCalcType(Handle: TLinkHandle): TCalcType; stdcall;
function GetFolderList(Handle: TLinkHandle; Buffer: PChar; Len: Integer): TCalcBool; stdcall;
function GetVarList(Handle: TLinkHandle; Folder: PChar; VarListCallBack: TVarListCallBack; ID: Pointer): TCalcBool;

{ Shell Folder Internal Functions }
function TryOpenConnection(out Handle: TLinkHandle; Window: HWnd): Boolean; stdcall;
function GetUIString(StringIdx: Integer): ShortString; stdcall;

function DllCanUnloadNow: HResult;

var
	UIStrings: TStringList;
	DLLRefCount: Integer = 0;

{$I StringInc}

implementation

uses
	SysUtils, Registry, ComServ;

const
	SendRetry = 1;
	SendTimeout = 300;
	ReceiveTimeout = 2000;
	SleepBetweenReady = 300;
	FirstReadyTimeout = 4000;
	NormalReadyTimeout = 3000;
	SendVarTimeout = 2000;
	ProgressInterval = 100;

type
	TLinkConnection = packed record
		cb: Word;
		Port: TLinkPort;
		Cable: TLinkType;
		Calc: TCalcType;
		Handle: THandle;
		Open: TCalcBool;
	end;
	PLinkConnection = ^TLinkConnection;

{$IFDEF LOGLINK}
var
	LogFile: Text;
	LogSend: Boolean;
{$ENDIF}

var
	OpenedOnce: Boolean = False;
	UserPort: TLinkPort = CALC_PORT_COM1;
	UserCable: TLinkType = CALC_CABLE_BLACKLINK;
	NonBlockingRead: Boolean = False;
	NonBlockingWrite: Boolean = True;

function GetPortState(Handle: THandle): Cardinal; forward;
procedure SetPortState(Handle: THandle; State: Cardinal); forward;
function GetPortName(PortID: TLinkPort): string; forward;
function IsValidAck(Value: Byte): Boolean; forward;

{	Creates a connection, opening the link port, but does not transfer any
	data. }
function CreateConnection(PortID: TLinkPort; CableType: TLinkType): TLinkHandle; stdcall;
var
	NewHandle: THandle;
	Settings: TDCB;
	Timeouts: TCommTimeouts;
begin
	Result := nil;
	if PortID = CALC_PORT_USER then
		PortID := UserPort;
	if CableType = CALC_CABLE_USER then
		CableType := UserCable;
	NewHandle := CreateFile (PChar (GetPortName (PortID) + ':'), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	if NewHandle = INVALID_HANDLE_VALUE then
		Exit;
	if CableType = CALC_CABLE_BLACKLINK then begin
		if not SetCommBreak (NewHandle) then begin
			CloseHandle (NewHandle);
			Exit;
		end;
	end else begin
		if not SetupComm (NewHandle, 1024, 1024) then begin
			CloseHandle (NewHandle);
			Exit;
		end;
		FillChar (Settings, SizeOf (Settings), 0);
		with Settings do begin
			DCBLength := SizeOf (Settings);
			BaudRate := CBR_9600;
			ByteSize := 8;
			StopBits := ONESTOPBIT;
			Flags := $00001001;
		end;
		if not SetCommState (NewHandle, Settings) then begin
			CloseHandle (NewHandle);
			Exit;
		end;
		FillChar (Timeouts, SizeOf (Timeouts), 0);
		with Timeouts do begin
			ReadIntervalTimeout := MaxDWord;
			if not NonBlockingRead then
				ReadTotalTimeoutConstant := ReceiveTimeout;
			if not NonBlockingWrite then
				WriteTotalTimeoutConstant := SendTimeOut;
		end;
		if not SetCommTimeouts (NewHandle, Timeouts) then begin
			CloseHandle (NewHandle);
			Exit;
		end;
	end;
	New (PLinkConnection (Result));
	FillChar (PLinkConnection(Result)^, SizeOf (TLinkConnection), 0);
	with PLinkConnection(Result)^ do begin
		cb := SizeOf (Result);
		Port := PortID;
		Cable := CableType;
		Handle := NewHandle;
	end;
end;

{	Opens a connection if it is not already open. }
function OpenConnection(Handle: TLinkHandle): TCalcBool;
var
	Timeout,
	StartTime: Cardinal;
begin
	if Assigned (Handle) then begin
		if PLinkConnection(Handle).Open then
			Result := True
		else begin
			{$IFDEF LOGLINK}
				AssignFile (LogFile, 'C:\Linklog.txt');
				ReWrite (LogFile);
				Write (LogFile, ' - Calculator Link Log - ');
				LogSend := False;
			{$ENDIF}
			PLinkConnection(Handle).Open := True;
			if OpenedOnce then
				Timeout := NormalReadyTimeout
			else
				Timeout := FirstReadyTimeout;
			StartTime := GetTickCount;
			repeat
				Result := CalcReady (Handle);
				if not Result then
					Sleep (SleepBetweenReady);
			until Result or (GetTickCount - StartTime > Timeout);
			OpenedOnce := True;
			if not Result then
				PLinkConnection(Handle).Open := False;
		end;
	end else
		Result := False;
end;

{	Closes a connection if it is open, and frees the handle. }
procedure CloseConnection(Handle: TLinkHandle); stdcall;
begin
	if Assigned (Handle) then begin
		if PLinkConnection(Handle).Cable = CALC_CABLE_BLACKLINK then
			SetPortState (PLinkConnection(Handle).Handle, 0);
		CloseHandle (PLinkConnection(Handle).Handle);
		Dispose (PLinkConnection (Handle));
		{$IFDEF LOGLINK}
			CloseFile (LogFile);
		{$ENDIF}
	end;
end;

procedure ResetConnection(Handle: TLinkHandle); stdcall;
begin
	if PLinkConnection(Handle).Cable = CALC_CABLE_BLACKLINK then
		SetPortState (PLinkConnection(Handle).Handle, 0);
end;

function WaitForAck(Handle: TLinkHandle; LastResult: PByte): TCalcBool; stdcall;
var
	TempResult: Byte;
begin
	Result :=
		ReceiveByte (Handle, TempResult) and IsValidAck (TempResult) and
		ReceiveByte (Handle, TempResult) and (TempResult = $56) and
		ReceiveByte (Handle, TempResult) and
		ReceiveByte (Handle, TempResult);
	if Assigned (LastResult) then
		LastResult^ := TempResult;
end;

function CalcReady(Handle: TLinkHandle): TCalcBool; stdcall;
var
	TempResult: Byte;
begin
	Result :=
		SendByte (Handle, $09) and
		SendByte (Handle, $68) and
		SendByte (Handle, $00) and
		SendByte (Handle, $00) and
		WaitForAck (Handle) and
		SendByte (Handle, $08) and
		SendByte (Handle, $68) and
		SendByte (Handle, $00) and
		SendByte (Handle, $00) and
		ReceiveByte (Handle, TempResult) and IsValidAck (TempResult);
	if not Result then
		Exit;
	case TempResult of
		$98: PLinkConnection(Handle).Calc := CALC_TYPE_TI89;
		$88: PLinkConnection(Handle).Calc := CALC_TYPE_TI92PLUS;
	end;
	Result :=
		ReceiveByte (Handle, TempResult) and (TempResult = $56) and
		ReceiveByte (Handle, TempResult) and
		ReceiveByte (Handle, TempResult);
end;

function CalcWaiting(Handle: TLinkHandle; LastResult: PByte): TCalcBool; stdcall;
var
	TempResult: Byte;
begin
	Result :=
		ReceiveByte (Handle, TempResult) and IsValidAck (TempResult) and
		ReceiveByte (Handle, TempResult) and (TempResult = $09) and
		ReceiveByte (Handle, TempResult) and
		ReceiveByte (Handle, TempResult);
	if Assigned (LastResult) then
		LastResult^ := TempResult;
end;

function SendAck(Handle: TLinkHandle): TCalcBool; stdcall;
begin
	Result :=
		SendByte (Handle, $08) and
		SendByte (Handle, $56) and
		SendByte (Handle, $00) and
		SendByte (Handle, $00);
end;

function SendWait(Handle: TLinkHandle): TCalcBool; stdcall;
begin
	Result :=
		SendByte (Handle, $08) and
		SendByte (Handle, $09) and
		SendByte (Handle, $00) and
		SendByte (Handle, $00);
end;

function SendByte(Handle: TLinkHandle; Value: Byte; CheckSum: PWord): TCalcBool; stdcall;
var
	I: Cardinal;
	StartTime: Cardinal;
	RetryCount: Integer;
begin
	Result := OpenConnection (Handle);
	if Result then begin
		{$IFDEF LOGLINK}
			if not LogSend then begin
				WriteLn (LogFile);
				WriteLn (LogFile, 'Sending:');
				LogSend := True;
			end;
			Write (LogFile, ' ' + IntToHex (Value, 2));
		{$ENDIF}
		if PLinkConnection(Handle).Cable = CALC_CABLE_GRAPHLINK then
			Result := WriteFile (PLinkConnection(Handle).Handle, Value, 1, I, nil)
		else begin
			StartTime := GetTickCount;
			for I := 0 to 7 do begin
				for RetryCount := 0 to SendRetry do begin
					if ((Value and (1 shl I)) <> 0) then
						SetPortState (PLinkConnection(Handle).Handle, 2)
					else
						SetPortState (PLinkConnection(Handle).Handle, 1);
					while GetPortState (PLinkConnection(Handle).Handle) <> 0 do
						if GetTickCount - StartTime > SendTimeout then
							Break;
					if GetPortState (PLinkConnection(Handle).Handle) = 0 then
						Break;
				end;
				if RetryCount > SendRetry then begin
					Result := False;
					Exit;
				end;
				for RetryCount := 0 to SendRetry do begin
					SetPortState (PLinkConnection(Handle).Handle, 3);
					while GetPortState (PLinkConnection(Handle).Handle) <> 3 do
						if GetTickCount - StartTime > SendTimeout then
							Break;
					if GetPortState (PLinkConnection(Handle).Handle) = 3 then
						Break;
				end;
				if RetryCount > SendRetry then begin
					Result := False;
					Exit;
				end;
			end;
		end;
		if Assigned (CheckSum) then
			Inc (CheckSum^, Value);
	end;
end;

function SendWord(Handle: TLinkHandle; Value: Word; CheckSum: PWord): TCalcBool; stdcall;
begin
	Result :=
		SendByte (Handle, Value, CheckSum) and
		SendByte (Handle, (Value shr (SizeOf (Value) * 4)), CheckSum);
end;

function SendLongWord(Handle: TLinkHandle; Value: LongWord; CheckSum: PWord): TCalcBool; stdcall;
begin
	Result :=
		SendWord (Handle, Value, CheckSum) and
		SendWord (Handle, (Value shr (SizeOf (Value) * 4)), CheckSum);
end;

function SendString(Handle: TLinkHandle; Value: PChar; CheckSum: PWord): TCalcBool; stdcall;
begin
	Result := True;
	while Value [0] <> #0 do begin
		Result := Result and SendByte (Handle, Byte (Value [0]), CheckSum);
		Value := @(Value[1]);
	end;
end;

function SendKey(Handle: TLinkHandle; Key: Char): TCalcBool; stdcall; overload;
begin
	Result :=
		SendByte (Handle, $09) and
		SendByte (Handle, $87) and
		SendWord (Handle, Byte (Key)) and
		WaitForAck (Handle);
end;

function SendKey(Handle: TLinkHandle; Key: Word): TCalcBool; stdcall; overload;
begin
	Result :=
		SendByte (Handle, $09) and
		SendByte (Handle, $87) and
		SendWord (Handle, Key) and
		WaitForAck (Handle);
end;

function SendKeys(Handle: TLinkHandle; Keys: PChar): TCalcBool; stdcall;
begin
	Result := OpenConnection (Handle);
	while Keys [0] <> #0 do begin
		Result := SendKey (Handle, Keys [0]);
		if not Result then
			Break;
		Keys := @(Keys[1]);
	end;
end;

function SendVar(Handle: TLinkHandle; Folder, Name: PChar; Buffer: Pointer; BufSize: Cardinal; Progress: PDWord; ProgressCallBack: TProgressCallBack; ID: Pointer): TCalcBool; stdcall;
var
	I: Cardinal;
	Buf: PByteArray;
	FileType: Byte;
	FileSize,
	BlockSize: Word;
	Sum: Word;
	TempProgress: DWord;
	StartTime: Cardinal;
begin
	Result := BufSize >= 88;
	if not Result then
		Exit;
	if not Assigned (Progress) then
		Progress := @TempProgress;
	Buf := Buffer;
	FileType := Buf [72];
	Buf [72] := 0;
	FileSize := Buf [86] shl 8 + Buf [87] + 2;
	if not Assigned (Folder) then
		Folder := @(Buf[10]);
	Folder [8] := #0;
	if not Assigned (Name) then
		Name := @(Buf[64]);
	Name [8] := #0;
	{$IFDEF LOGLINK}
		WriteLn (LogFile);
		WriteLn (LogFile);
		WriteLn (LogFile, AnsiString (Folder), '\', AnsiString (Name), ': ', FileSize, ' (', IntToHex (FileSize, 4), ') Bytes');
	{$ENDIF}
	Sum := 0;
	StartTime := GetTickCount;
	repeat until CalcReady (Handle) or (GetTickCount - StartTime > SendVarTimeout);
	Result :=
		SendKey (Handle, 264) and
		SendKey (Handle, 264) and
		SendKey (Handle, 277) and
		SendByte (Handle, $08) and
		SendByte (Handle, $06) and
		SendWord (Handle, StrLen (Folder) + StrLen (Name) + 8) and
		SendLongWord (Handle, FileSize, @Sum) and
		SendByte (Handle, FileType, @Sum) and
		SendByte (Handle, StrLen (Folder) + StrLen (Name) + 1, @Sum) and
		SendString (Handle, Folder, @Sum) and
		SendString (Handle, '\', @Sum) and
		SendString (Handle, Name, @Sum) and
		SendByte (Handle, $00) and
		SendWord (Handle, Sum) and
		WaitForAck (Handle) and
		CalcWaiting (Handle) and
		SendAck (Handle) and
		SendByte (Handle, $08) and
		SendByte (Handle, $15) and
		SendWord (Handle, FileSize + 4) and
		SendLongWord (Handle, 0);
	if not Result then
		Exit;
	BlockSize := FileSize - 2;
	Sum := 0;
	Result :=
		SendByte (Handle, BlockSize and $FF00 shr 8, @Sum) and
		SendByte (Handle, BlockSize and $00FF, @Sum);
	Inc (Progress^, 2);
	if not Result then
		Exit;
	for I := 0 to BlockSize - 1 do begin
		if (I < BufSize) and SendByte (Handle, Buf [I + 88], @Sum) then
			Inc (Progress^)
		else begin
			Result := False;
			Break;
		end;
		if ((I + 2) mod ProgressInterval = 0) and Assigned (ProgressCallBack) and (not ProgressCallBack (ID, Progress)) then begin
			Result := False;
			Break;
		end;
	end;
	if not Result then
		Exit;
	Result :=
		SendWord (Handle, Sum) and
		WaitForAck (Handle) and
		SendByte (Handle, $08) and
		SendByte (Handle, $92) and
		SendByte (Handle, $00) and
		SendByte (Handle, $00) and
		WaitForAck (Handle);
end;

function SendFile(Handle: TLinkHandle; Folder, Name: PChar; FileName: PChar; Progress: PDWord; ProgressCallBack: TProgressCallBack; ID: Pointer): TCalcBool; stdcall;
var
	FHandle: HFile;
	MHandle: HGlobal;
	Buffer: Pointer;
	Size,
	Read: Cardinal;
begin
	FHandle := CreateFile (FileName, GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, 0);
	Result := FHandle <> 0;
	if not Result then
		Exit;
	Size := GetFileSize (FHandle, nil);
	MHandle := GlobalAlloc (GPTR, Size);
	Result := MHandle <> 0;
	if not Result then begin
		CloseHandle (FHandle);
		Exit;
	end;
	Buffer := GlobalLock (MHandle);
	Result := ReadFile (FHandle, Buffer^, Size, Read, nil);
	CloseHandle (FHandle);
	if Result then
		Result := SendVar (Handle, Folder, Name, Buffer, Size, Progress, ProgressCallBack, ID);
	GlobalFree (MHandle);
end;

function CheckFileFormat(Handle: TLinkHandle; FileName: PChar; Folder, Name: PChar; Size: PWord): TCalcBool; stdcall;
var
	FHandle: HFile;
	Buffer: array [0..4] of Char;
	Read: Cardinal;
begin
	FHandle := CreateFile (FileName, GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
	Result :=
		(FHandle <> 0) and
		ReadFile (FHandle, Buffer, 4, Read, nil);
	if not Result then
		Exit;
	Buffer [4] := #0;
	Result := Buffer = '**TI';
	if not Result then
		Exit;
	if Assigned (Folder) then begin
		SetFilePointer (FHandle, 10, nil, FILE_BEGIN);
		Result := ReadFile (FHandle, Folder^, 8, Read, nil) and Result;
		Folder [8] := #0;
	end;
	if Assigned (Name) then begin
		SetFilePointer (FHandle, 64, nil, FILE_BEGIN);
		Result := ReadFile (FHandle, Name^, 8, Read, nil) and Result;
		Name [8] := #0;
	end;
	if Assigned (Size) then begin
		SetFilePointer (FHandle, 86, nil, FILE_BEGIN);
		Result := ReadFile (FHandle, Buffer, 2, Read, nil) and Result;
		Size^ := Byte (Buffer [0]) shl 8 + Byte (Buffer [1]) + 2;
	end;
	if FHandle <> 0 then
		CloseHandle (FHandle);
end;

function ExecuteHomeLine(Handle: TLinkHandle; Line: PChar): TCalcBool; stdcall;
begin
	Result :=
		SendKey (Handle, 264) and
		SendKey (Handle, 264) and
		SendKey (Handle, 277) and
		SendKey (Handle, 263) and
		SendKey (Handle, 263) and
		SendKeys (Handle, Line) and
		SendKey (Handle, 13);
end;

function ReceiveByte(Handle: TLinkHandle; out Value: Byte; CheckSum: PWord = nil): TCalcBool; stdcall;
var
	I,
	State: Cardinal;
	StartTime: Cardinal;
begin
	Value := 0;
	Result := OpenConnection (Handle);
	if Result then begin
		if PLinkConnection(Handle).Cable = CALC_CABLE_GRAPHLINK then begin
			StartTime := GetTickCount;
			repeat
				Result := ReadFile (PLinkConnection(Handle).Handle, Value, 1, I, nil);
			until Result or (GetTickCount - StartTime > ReceiveTimeout);
		end else begin
			StartTime := GetTickCount;
			for I := 0 to 7 do begin
				repeat
					State := GetPortState (PLinkConnection(Handle).Handle);
					if (State = 3) and (GetTickCount - StartTime > ReceiveTimeout) then begin
						Result := False;
						Exit;
					end;
				until State <> 3;
				if State = 1 then begin
					Value := Value or (1 shl I);
					SetPortState (PLinkConnection(Handle).Handle, 1);
					State := 2;
				end else begin
					SetPortState (PLinkConnection(Handle).Handle, 2);
					State := 1;
				end;
				while (GetPortState (PLinkConnection(Handle).Handle) and State) = 0 do
					if GetTickCount - StartTime > ReceiveTimeout then begin
						Result := False;
						Exit;
					end;
				SetPortState (PLinkConnection(Handle).Handle, 3);
			end;
		end;
		if Assigned (CheckSum) then
			Inc (CheckSum^, Value);
		{$IFDEF LOGLINK}
			if LogSend then begin
				WriteLn (LogFile);
				WriteLn (LogFile, 'Receiving:');
				LogSend := False;
			end;
			Write (LogFile, ' ' + IntToHex (Value, 2));
		{$ENDIF}
	end;
end;

function ReceiveWord(Handle: TLinkHandle; out Value: Word; CheckSum: PWord = nil): TCalcBool; stdcall;
var
	Temp1,
	Temp2: Byte;
begin
	Result :=
		ReceiveByte (Handle, Temp1, CheckSum) and
		ReceiveByte (Handle, Temp2, CheckSum);
	Value := Temp1 or (Temp2 shl (SizeOf (Temp1) * 8));
end;

function ReceiveLongWord(Handle: TLinkHandle; out Value: LongWord; CheckSum: PWord = nil): TCalcBool; stdcall;
var
	Temp1,
	Temp2: Word;
begin
	Result :=
		ReceiveWord (Handle, Temp1, CheckSum) and
		ReceiveWord (Handle, Temp2, CheckSum);
	Value := Temp1 or (Temp2 shl (SizeOf (Temp1) * 8));
end;

function GetCalcType(Handle: TLinkHandle): TCalcType; stdcall;
begin
	if OpenConnection (Handle) then begin
		Result := PLinkConnection(Handle).Calc;
	end else
		Result := CALC_TYPE_ERROR;
end;

function GetFolderList(Handle: TLinkHandle; Buffer: PChar; Len: Integer): TCalcBool; stdcall;
var
	BufEnd,
	TempBuf: PChar;
	I,
	J: Integer;
	Data: Byte;
	LongData: LongWord;
	Sum,
	CheckSum,
	BlockSize: Word;
	NameLength: Byte;
begin
	BufEnd := @(Buffer[Len-1]);
	Sum := 0;
	Result :=
		SendByte (Handle, $08) and
		SendByte (Handle, $A2) and
		SendByte (Handle, $06) and
		SendLongWord (Handle, 0) and
		SendByte (Handle, $1F, @Sum) and
		SendByte (Handle, $1A, @Sum) and
		SendByte (Handle, $00) and
		SendWord (Handle, Sum) and
		WaitForAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and (Data = $06) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data);
	if not Result then
		Exit;
	Sum := 0;
	Result :=
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveByte (Handle, Data, @Sum) and
		ReceiveByte (Handle, NameLength, @Sum);
	if not Result then
		Exit;
	for I := 0 to NameLength - 1 do
		Result := Result and ReceiveByte (Handle, Data, @Sum);
	if not Result then
		Exit;
	Result :=
		ReceiveByte (Handle, Data, @Sum) and
		ReceiveWord (Handle, CheckSum) and (CheckSum = Sum) and
		SendAck (Handle) and
		SendWait (Handle) and
		WaitForAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and (Data = $15) and
		ReceiveWord (Handle, BlockSize);
	if not Result then
		Exit;
	Sum := 0;
	Result := ReceiveLongWord (Handle, LongData, @Sum);
	if not Result then
		Exit;
	Dec (BlockSize, 4);
	for J := 0 to BlockSize div 14 - 1 do begin
		TempBuf := Buffer;
		for I := 0 to 7 do begin
			Result := ReceiveByte (Handle, Data, @Sum);
			if not Result then begin
				Buffer [0] := #0;
				Exit;
			end;
			if (Data <> 0) and (Buffer <> BufEnd) then begin
				{$IFDEF LOGLINK}
					Write (LogFile, ':' + Char (Data));
				{$ENDIF}
				Buffer [0] := Char (Data);
				Buffer := @(Buffer[1]);
			end;
		end;
		Result := ReceiveByte (Handle, Data, @Sum);
		if not Result then
			Exit;
		if Data = $1F then begin
			if Buffer <> BufEnd then begin
				Buffer [0] := ';';
				Buffer := @(Buffer[1]);
			end;
		end else
			Buffer := TempBuf;
		Result :=
			ReceiveByte (Handle, Data, @Sum) and
			ReceiveLongWord (Handle, LongData, @Sum);
		if not Result then
			Exit;
	end;
	Buffer [0] := #0;
	Result :=
		ReceiveWord (Handle, CheckSum) and (CheckSum = Sum) and
		SendAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data) and
		SendAck (Handle);
end;

function GetVarList(Handle: TLinkHandle; Folder: PChar; VarListCallBack: TVarListCallBack; ID: Pointer): TCalcBool;
var
	I,
	J: Integer;
	Data: Byte;
	WordData: Word;
	LongData: LongWord;
	Sum,
	CheckSum,
	BlockSize: Word;
	NameLength: Byte;
	CurVar: TCalcVar;
begin
	Sum := 0;
	Result :=
		Assigned (Folder) and
		SendByte (Handle, $08) and
		SendByte (Handle, $A2) and
		SendWord (Handle, StrLen (Folder) + 6) and
		SendWord (Handle, 0) and
		SendByte (Handle, 0) and
		SendByte (Handle, $1B, @Sum) and
		SendByte (Handle, $1A, @Sum) and
		SendByte (Handle, StrLen (Folder), @Sum) and
		SendString (Handle, Folder, @Sum) and
		SendWord (Handle, Sum) and
		WaitForAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and (Data = $06) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data);
	if not Result then
		Exit;
	Sum := 0;
	Result :=
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveByte (Handle, Data, @Sum) and
		ReceiveByte (Handle, NameLength, @Sum);
	if not Result then
		Exit;
	for I := 0 to NameLength - 1 do
		Result := Result and ReceiveByte (Handle, Data, @Sum);
	Result :=
		ReceiveByte (Handle, Data, @Sum) and
		ReceiveWord (Handle, CheckSum) and (CheckSum = Sum) and
		SendAck (Handle) and
		SendWait (Handle) and
		WaitForAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and (Data = $15) and
		ReceiveWord (Handle, BlockSize);
	if not Result then
		Exit;
	Sum := 0;
	Result :=
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveLongWord (Handle, LongData, @Sum) and
		ReceiveWord (Handle, WordData, @Sum);
	if not Result then
		Exit;
	Dec (BlockSize, 18);
	for J := 0 to BlockSize div 14 - 1 do begin
		CurVar.Version := 1;
		CurVar.CalcType := PLinkConnection(Handle).Calc;
		StrLCopy (CurVar.Folder, Folder, 9);
		FillChar (CurVar.Name, SizeOf (CurVar.Name), 0);
		for I := 0 to 7 do begin
			Result := ReceiveByte (Handle, Data, @Sum);
			if not Result then
				Exit;
			if (Data <> 0) then begin
				{$IFDEF LOGLINK}
					Write (LogFile, ':' + Char (Data));
				{$ENDIF}
				CurVar.Name [I] := Char (Data);
			end;
		end;
		Result :=
			ReceiveByte (Handle, CurVar.TypeID, @Sum) and
			ReceiveByte (Handle, CurVar.Attr, @Sum) and
			ReceiveLongWord (Handle, CurVar.Size, @Sum);
		if not Result then
			Exit;
		CurVar.Size := CurVar.Size and $FFFFFF;
		if CurVar.TypeID <> $24 then
			Result := VarListCallBack (ID, CurVar);
		if not Result then
			Exit;
	end;
	Result :=
		ReceiveWord (Handle, CheckSum) and (CheckSum = Sum) and
		SendAck (Handle) and
		ReceiveByte (Handle, Data) and IsValidAck (Data) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data) and
		ReceiveByte (Handle, Data) and
		SendAck (Handle);
end;

function TryOpenConnection(out Handle: TLinkHandle; Window: HWnd): Boolean;
begin
	Handle := CreateConnection (CALC_PORT_USER, CALC_CABLE_USER);
	if Assigned (Handle) then begin
		Result := OpenConnection (Handle);
		if not Result then begin
			CloseConnection (Handle);
			Handle := nil;
			MessageBox (Window, PChar (UIStrings [UILinkError]), PChar (UIStrings [UIError]), MB_OK or MB_ICONERROR);
		end;
	end else begin
		Result := False;
		MessageBox (Window, PChar (UIStrings [UIOpenError]), PChar (UIStrings [UIError]), MB_OK or MB_ICONERROR);
	end;
end;

function GetPortState(Handle: THandle): Cardinal;
var
	InternalState: Cardinal;
begin
	GetCommModemStatus (Handle, InternalState);
	Result := 0;
	if (InternalState and MS_CTS_ON) <> 0 then
		Inc (Result, 1);
	if (InternalState and MS_DSR_ON) <> 0 then
		Inc (Result, 2);
end;

procedure SetPortState(Handle: THandle; State: Cardinal);
procedure Send(Func: Cardinal);
begin
	EscapeCommFunction (Handle, Func);
end;
begin
	if (State and 2) <> 0 then
		Send (SETRTS)
	else
		Send (CLRRTS);
	if (State and 1) <> 0 then
		Send (SETDTR)
	else
		Send (CLRDTR);
end;

function GetPortName(PortID: TLinkPort): string;
begin
	if (PortID and CALC_PORT_COM_MASK) <> 0 then
		Result := 'COM' + IntToStr (PortID - CALC_PORT_COM_MASK)
	else if (PortID and CALC_PORT_LPT_MASK) <> 0 then
		Result := 'LPT' + IntToStr (PortID - CALC_PORT_LPT_MASK)
	else
		Result := 'COM1';
end;

function IsValidAck(Value: Byte): Boolean;
begin
	Result := Value in [$88, $89, $98];
end;

function DllCanUnloadNow: HResult;
begin
	Result := ComServ.DllCanUnloadNow;
	if DLLRefCount > 0 then
		Result := S_False;
end;

function GetUIString(StringIdx: Integer): ShortString;
begin
	if StringIdx < UIStrings.Count then
		Result := UIStrings [StringIdx]
	else
		Result := '';
end;

var
	FileName: array [0..Max_Path] of Char;

initialization
	try
		with TRegistry.Create do try
			RootKey := HKEY_CURRENT_USER;
			if OpenKeyReadOnly ('Software\SeReSoft\FolderLink') then begin
				if ValueExists ('Port') then
					UserPort := ReadInteger ('Port');
				if ValueExists ('Cable') then
					UserCable := ReadInteger ('Cable');
				if ValueExists ('COMNonBlockingRead') then
					NonBlockingRead := ReadBool ('COMNonBlockingRead');
				if ValueExists ('COMNonBlockingWrite') then
					NonBlockingWrite := ReadBool ('COMNonBlockingWrite');
			end;
		finally
			Free;
		end;
	except end;
	try
		GetModuleFileName (hInstance, FileName, Max_Path + 1);
		UIStrings := TStringList.Create;
		UIStrings.LoadFromFile (ChangeFileExt (AnsiString (FileName), '.str'));
	except
		MessageBox (0, 'Error getting string list.', 'Error', MB_OK or MB_ICONERROR);
		Abort;
	end;
finalization
	UIStrings.Free;
end.
