unit ShellFolderUnit;

interface

uses
	LinkUnit, SubFolderUnit,
	Windows, SysUtils, Classes, ActiveX, ComObj, ShlObj;

type
	TShellFolder = class(TComObject, IPersist, IPersistFolder, IShellFolder)
	protected
		function GetClassID(out classID: TCLSID): HResult; virtual; stdcall;
		function IPersistFolder.Initialize = InitializeFolder;
		function InitializeFolder(pidl: PItemIDList): HResult; virtual; stdcall;
		function ParseDisplayName(hwndOwner: HWND;
			pbcReserved: Pointer; lpszDisplayName: POLESTR; out pchEaten: ULONG;
			out ppidl: PItemIDList; var dwAttributes: ULONG): HResult; virtual; stdcall;
		function EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
			out EnumIDList: IEnumIDList): HResult; virtual; stdcall;
		function BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
			const riid: TIID; out ppvOut: Pointer): HResult; virtual; stdcall;
		function BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
			const riid: TIID; out ppvObj: Pointer): HResult; virtual; stdcall;
		function CompareIDs(lParam: LPARAM;
			pidl1, pidl2: PItemIDList): HResult; virtual; stdcall;
		function CreateViewObject(hwndOwner: HWND; const riid: TIID;
			out ppvOut: Pointer): HResult; virtual; stdcall;
		function GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
			var rgfInOut: UINT): HResult; virtual; stdcall;
		function GetUIObjectOf(hwndOwner: HWND; cidl: UINT; var apidl: PItemIDList;
			const riid: TIID; prgfInOut: Pointer; out ppvOut: Pointer): HResult; virtual; stdcall;
		function GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
			var lpName: TStrRet): HResult; virtual; stdcall;
		function SetNameOf(hwndOwner: HWND; pidl: PItemIDList; lpszName: POLEStr;
			uFlags: DWORD; var ppidlOut: PItemIDList): HResult; virtual; stdcall;
	end;

const
	Class_ShellFolder: TGUID = '{595D2820-10B5-11D5-A253-00900001489D}';

implementation

uses
	ComServ,
	ShellViewUnit, EnumIDListUnit, ExtractIconUnit, DropTargetUnit;

{ TShellFolder }

function TShellFolder.BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
	const riid: TIID; out ppvOut: Pointer): HResult;
var
	Item: PSubFolderList;
	IDIndex: Integer;
	Intf: IShellFolder;
begin
	Item := PSubFolderList (pidl);
	if Item^[0].cb > 0 then begin
		IDIndex := 0;
		while Item^[IDIndex+1].cb > 0 do
			Inc (IDIndex);
		if GUIDToString (riid) = SID_IShellFolder then begin
			Intf := TSubFolder.Create (Item^[IDIndex]);
			Result := Intf.QueryInterface (riid, ppvOut);
		end else
			Result := E_NoInterface;
	end else
		Result := E_Fail;
end;

function TShellFolder.BindToStorage(pidl: PItemIDList;
	pbcReserved: Pointer; const riid: TIID; out ppvObj: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TShellFolder.CompareIDs(lParam: LPARAM;
	pidl1, pidl2: PItemIDList): HResult;
var
	Item: PSubFolderList;
	IDIndex: Integer;
	Name1,
	Name2: string;
begin
	Item := PSubFolderList (pidl1);
	if Item^[0].cb > 0 then begin
		IDIndex := 0;
		while Item^[IDIndex+1].cb > 0 do
			Inc (IDIndex);
		Name1 := Item^[IDIndex].Name;
		if Length (Name1) < 9 then
			Name1 [Length (Name1) + 1] := #0;
		Item := PSubFolderList (pidl2);
		if Item^[0].cb > 0 then begin
			IDIndex := 0;
			while Item^[IDIndex+1].cb > 0 do
				Inc (IDIndex);
			Name2 := Item^[IDIndex].Name;
			if Length (Name2) < 9 then
				Name2 [Length (Name2) + 1] := #0;
			if Name1 = Name2 then
				Result := 0
			else
				Result := AnsiCompareStr (Name1, Name2);
		end else
			Result := 0;
	end else
		Result := 0;
end;

function TShellFolder.CreateViewObject(hwndOwner: HWND; const riid: TIID;
	out ppvOut: Pointer): HResult;
var
	Intf: IShellView;
begin
	if GUIDToString (riid) = SID_IShellView then begin
		Intf := TShellView.Create;
		Result := Intf.QueryInterface (riid, ppvOut);
	end else
		Result := E_NoInterface;
end;

function TShellFolder.EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
	out EnumIDList: IEnumIDList): HResult;
var
	Intf: IEnumIDList;
	Items: TSubFolderItems;
begin
	if GetFolders (hwndOwner, Items) then begin
		Intf := TEnumIDList.Create (Items);
		Result := Intf.QueryInterface (IEnumIDList, EnumIDList);
	end else
		Result := E_Fail;
end;

function TShellFolder.GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
	var rgfInOut: UINT): HResult;
begin
	rgfInOut := SFGAO_CANCOPY or SFGAO_CANDELETE or SFGAO_CANRENAME or SFGAO_DROPTARGET or SFGAO_FOLDER or SFGAO_REMOVABLE;
	Result := S_OK;
end;

function TShellFolder.GetClassID(out classID: TCLSID): HResult;
begin
	ClassID := Class_ShellFolder;
	Result := S_OK;
end;

function TShellFolder.GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
	var lpName: TStrRet): HResult;
var
	Item: PSubFolderList;
	IDIndex: Integer;
begin
	Item := PSubFolderList (pidl);
	if Item^[0].cb > 0 then begin
		IDIndex := 0;
		while Item^[IDIndex+1].cb > 0 do
			Inc (IDIndex);
		lpName.uType := STRRET_CSTR;
		if uFlags = SHGDN_FORPARSING then
			StrPCopy (lpName.cStr, 'Calculator\' + Item^[IDIndex].Name)
		else
			StrPCopy (lpName.cStr, Item^[IDIndex].Name);
		Result := S_OK;
	end else
		Result := E_Fail;
end;

function TShellFolder.GetUIObjectOf(hwndOwner: HWND; cidl: UINT;
	var apidl: PItemIDList; const riid: TIID; prgfInOut: Pointer;
	out ppvOut: Pointer): HResult;
var
	Item: PSubFolderList;
	IDIndex: Integer;
	Intf: IUnknown;
begin
	if GUIDToString (riid) = SID_IExtractIconA then begin
		Intf := TExtractIcon.Create;
		Result := Intf.QueryInterface (riid, ppvOut);
	end else if GUIDToString (riid) = GUIDToString (IDropTarget) then begin
		Item := PSubFolderList (apidl);
		if Item^[0].cb > 0 then begin
			IDIndex := 0;
			while Item^[IDIndex+1].cb > 0 do
				Inc (IDIndex);
			Intf := TDropTarget.Create (Item^[IDIndex].Name);
			Result := Intf.QueryInterface (riid, ppvOut);
		end else
			Result := E_NotImpl;
	end else
		Result := E_NoInterface;
end;

function TShellFolder.InitializeFolder(pidl: PItemIDList): HResult;
begin
	Result := S_OK;
end;

function TShellFolder.ParseDisplayName(hwndOwner: HWND;
	pbcReserved: Pointer; lpszDisplayName: POLESTR; out pchEaten: ULONG;
	out ppidl: PItemIDList; var dwAttributes: ULONG): HResult;
begin
	Result := E_NotImpl;
end;

function TShellFolder.SetNameOf(hwndOwner: HWND; pidl: PItemIDList;
	lpszName: POLEStr; uFlags: DWORD; var ppidlOut: PItemIDList): HResult;
begin
	Result := E_NotImpl;
end;

initialization
	TComObjectFactory.Create(ComServer, TShellFolder, Class_ShellFolder,
		'ShellFolder', 'My Calculator', ciMultiInstance, tmApartment);
end.
