unit ShellViewUnit;

interface

uses
	LinkUnit, SubFolderUnit,
	Windows, SysUtils, ActiveX, ComObj, ShlObj, CommCtrl;

type
	TShellView = class(TComObject, IOleWindow, IShellView)
	private
		FHandle: HWnd;
		FBrowser: IShellBrowser;
		FItems: TSubFolderItems;
	protected
		function GetWindow(out wnd: HWnd): HResult; virtual; stdcall;
		function ContextSensitiveHelp(fEnterMode: BOOL): HResult; virtual; stdcall;
		function TranslateAccelerator(var Msg: TMsg): HResult; virtual; stdcall;
		function EnableModeless(Enable: Boolean): HResult; virtual; stdcall;
		function UIActivate(State: UINT): HResult; virtual; stdcall;
		function Refresh: HResult; virtual; stdcall;
		function CreateViewWindow(PrevView: IShellView;
			var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
			var Rect: TRect; out Wnd: HWND): HResult; virtual; stdcall;
		function DestroyViewWindow: HResult; virtual; stdcall;
		function GetCurrentInfo(out FolderSettings: TFolderSettings): HResult; virtual; stdcall;
		function AddPropertySheetPages(Reseved: DWORD;
			lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult; virtual; stdcall;
		function SaveViewState: HResult; virtual; stdcall;
		function SelectItem(pidl: PItemIDList; flags: UINT): HResult; virtual; stdcall;
		function GetItemObject(Item: UINT; const iid: TIID; IPtr: Pointer): HResult; virtual; stdcall;
		property Handle: HWND read FHandle;
	public
		constructor Create;
		destructor Destroy; override;
	end;

const
	Class_ShellView: TGUID = '{595D2821-10B5-11D5-A253-00900001489D}';

function GetFolders(Window: HWnd; out Items: TSubFolderItems): Boolean;

implementation

uses
	ComServ, ShellAPI;

function GetFolders(Window: HWnd; out Items: TSubFolderItems): Boolean;
var
	Link: TLinkHandle;
	Buf: array [0..MAX_PATH] of Char;
	S: string;
begin
	SetLength (Items, 0);
	if TryOpenConnection (Link, Window) then begin
		if GetFolderList (Link, Buf, MAX_PATH + 1) then begin
			S := AnsiString (Buf);
			while Pos (';', S) > 0 do begin
				SetLength (Items, Length (Items) + 1);
				with Items [High (Items)] do begin
					cb := SizeOf (TSubFolderItem);
					Name := Copy (S, 1, Pos (';', S) - 1);
				end;
				Delete (S, 1, Pos (';', S));
			end;
			Result := True;
		end else begin
			MessageBox (Window, PChar (UIStrings [UIFolderListError]), PChar (UIStrings [UIError]), MB_OK or MB_ICONERROR);
			Result := False;
		end;
		CloseConnection (Link);
	end else
		Result := False;
end;

{ TShellView }

function TShellView.AddPropertySheetPages(Reseved: DWORD;
	lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.ContextSensitiveHelp(fEnterMode: BOOL): HResult;
begin
	Result := E_NotImpl;
end;

constructor TShellView.Create;
begin
	inherited;
	Inc (DllRefCount);
end;

function TShellView.CreateViewWindow(PrevView: IShellView;
	var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
	var Rect: TRect; out Wnd: HWND): HResult;
var
	ParentWindow: HWnd;
	ColData: TLVColumn;
	FileInfo: TSHFileInfo;
begin
	FBrowser := ShellBrowser;
	FBrowser.GetWindow (ParentWindow);
	FHandle := CreateWindowEx (WS_EX_CLIENTEDGE, WC_LISTVIEW, 'Calculator', WS_VISIBLE or WS_CHILD or WS_CLIPSIBLINGS or LVS_REPORT or LVS_SHAREIMAGELISTS, Rect.Left, Rect.top, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top, ParentWindow, 0, 0, nil);
	Wnd := Handle;
	ListView_SetImageList (Handle, SHGetFileInfo ('C:\', 0, FileInfo, SizeOf (FileInfo), SHGFI_SYSICONINDEX or SHGFI_SMALLICON), LVSIL_SMALL);
	ListView_SetImageList (Handle, SHGetFileInfo ('C:\', 0, FileInfo, SizeOf (FileInfo), SHGFI_SYSICONINDEX or SHGFI_LARGEICON), LVSIL_NORMAL);
	with ColData do begin
		Mask := LVCF_FMT or LVCF_WIDTH or LVCF_TEXT;
		Fmt := LVCFMT_LEFT;
		cX := 150;
		pszText := PChar (UIStrings [UIFolderName]);
	end;
	ListView_InsertColumn (Handle, 0, ColData);
	Result := Refresh;
end;

destructor TShellView.Destroy;
begin
	Dec (DllRefCount);
	inherited;
end;

function TShellView.DestroyViewWindow: HResult;
begin
	if FHandle <> 0 then begin
		DestroyWindow (FHandle);
		FHandle := 0;
	end;
	Result := S_OK;
end;

function TShellView.EnableModeless(Enable: Boolean): HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.GetCurrentInfo(out FolderSettings: TFolderSettings):
	HResult;
begin
	with FolderSettings do begin
		ViewMode := FVM_DETAILS;
		fFlags := 0;
	end;
	Result := S_OK;
end;

function TShellView.GetItemObject(Item: UINT; const iid: TIID;
	IPtr: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.GetWindow(out wnd: HWnd): HResult;
begin
	Wnd := Handle;
	Result := S_OK;
end;

function TShellView.Refresh: HResult;
const
	Widths: array [0..0] of Integer = (-1);
var
	I: Integer;
	Item: TLVItem;
	S: string;
begin
	ListView_DeleteAllItems (Handle);
	if GetFolders (0, FItems) then begin
		with Item do begin
			iItem := 0;
			iSubItem := 0;
			iImage := 3;
			Mask := LVIF_TEXT or LVIF_IMAGE;
		end;
		for I := Low (FItems) to High (FItems) do begin
			Item.pszText := PChar (AnsiString (FItems[I].Name));
			ListView_InsertItem (Handle, Item);
			Inc (Item.iItem);
		end;
		FBrowser.SendControlMsg (FCW_STATUS, SB_SETPARTS, 1, Cardinal (@Widths), 0);
		if Length (FItems) = 1 then
			S := UIStrings [UIFolderCount1]
		else
			S := Format (UIStrings [UIFolderCount], [Length (FItems)]);
		FBrowser.SendControlMsg (FCW_STATUS, SB_SETTEXT, 0, Cardinal (PChar (S)), 0);
		Result := S_OK;
	end else
		Result := E_Fail;
end;

function TShellView.SaveViewState: HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.SelectItem(pidl: PItemIDList; flags: UINT): HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.TranslateAccelerator(var Msg: TMsg): HResult;
begin
	Result := E_NotImpl;
end;

function TShellView.UIActivate(State: UINT): HResult;
begin
	Result := S_OK;
end;

initialization
	TComObjectFactory.Create(ComServer, TShellView, Class_ShellView,
		'ShellView', '', ciInternal, tmApartment);
end.
