unit SubFolderUnit;

interface

uses
	LinkUnit,
	Windows, Classes, ActiveX, ComObj, ShlObj;

type
	TSubFolderName = string [9];

	TSubFolderItem = packed record
		cb: Word;
		Name: TSubFolderName;
	end;
	PSubFolderItem = ^TSubFolderItem;
	TSubFolderItems = array of TSubFolderItem;
	PSubFolderItems = ^TSubFolderItems;

	TSubFolderList = array [0..MaxListSize] of TSubFolderItem;
	PSubFolderList = ^TSubFolderList;

	TVarItems = array of TCalcVar;
	PVarItems = ^TVarItems;

	TSubFolder = class(TComObject, IPersist, IPersistFolder, IShellFolder)
	private
		FData: TSubFolderItem;
	protected
		function GetClassID(out classID: TCLSID): HResult; virtual; stdcall;
		function IPersistFolder.Initialize = InitializeFolder;
		function InitializeFolder(pidl: PItemIDList): HResult; virtual; stdcall;
		function ParseDisplayName(hwndOwner: HWND;
			pbcReserved: Pointer; lpszDisplayName: POLESTR; out pchEaten: ULONG;
			out ppidl: PItemIDList; var dwAttributes: ULONG): HResult; virtual; stdcall;
		function EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
			out EnumIDList: IEnumIDList): HResult; virtual; stdcall;
		function BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
			const riid: TIID; out ppvOut: Pointer): HResult; virtual; stdcall;
		function BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
			const riid: TIID; out ppvObj: Pointer): HResult; virtual; stdcall;
		function CompareIDs(lParam: LPARAM;
			pidl1, pidl2: PItemIDList): HResult; virtual; stdcall;
		function CreateViewObject(hwndOwner: HWND; const riid: TIID;
			out ppvOut: Pointer): HResult; virtual; stdcall;
		function GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
			var rgfInOut: UINT): HResult; virtual; stdcall;
		function GetUIObjectOf(hwndOwner: HWND; cidl: UINT; var apidl: PItemIDList;
			const riid: TIID; prgfInOut: Pointer; out ppvOut: Pointer): HResult; virtual; stdcall;
		function GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
			var lpName: TStrRet): HResult; virtual; stdcall;
		function SetNameOf(hwndOwner: HWND; pidl: PItemIDList; lpszName: POLEStr;
			uFlags: DWORD; var ppidlOut: PItemIDList): HResult; virtual; stdcall;
	public
		constructor Create(const Data: TSubFolderItem);
		destructor Destroy; override;
	end;

const
	Class_SubFolder: TGUID = '{595D2823-10B5-11D5-A253-00900001489D}';

implementation

uses
	SubViewUnit,
	ComServ;

{ TSubFolder }

function TSubFolder.BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
	const riid: TIID; out ppvOut: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
	const riid: TIID; out ppvObj: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.CompareIDs(lParam: LPARAM;
	pidl1, pidl2: PItemIDList): HResult;
begin
	Result := E_NotImpl;
end;

constructor TSubFolder.Create(const Data: TSubFolderItem);
begin
	inherited Create;
	Inc (DLLRefCount);
	FData := Data;
end;

function TSubFolder.CreateViewObject(hwndOwner: HWND; const riid: TIID;
	out ppvOut: Pointer): HResult;
var
	Intf: IShellView;
begin
	if GUIDToString (riid) = SID_IShellView then begin
		Intf := TSubView.Create (FData);
		Result := Intf.QueryInterface (riid, ppvOut);
	end else
		Result := E_NoInterface;
end;

destructor TSubFolder.Destroy;
begin
	Dec (DLLRefCount);
	inherited;
end;

function TSubFolder.EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
	out EnumIDList: IEnumIDList): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
	var rgfInOut: UINT): HResult;
begin
	rgfInOut := SFGAO_CANDELETE or SFGAO_FOLDER or SFGAO_REMOVABLE;
	Result := S_OK;
end;

function TSubFolder.GetClassID(out classID: TCLSID): HResult;
begin
	ClassID := Class_SubFolder;
	Result := S_OK;
end;

function TSubFolder.GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
	var lpName: TStrRet): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.GetUIObjectOf(hwndOwner: HWND; cidl: UINT;
	var apidl: PItemIDList; const riid: TIID; prgfInOut: Pointer;
	out ppvOut: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.InitializeFolder(pidl: PItemIDList): HResult;
begin
	Result := S_OK;
end;

function TSubFolder.ParseDisplayName(hwndOwner: HWND; pbcReserved: Pointer;
	lpszDisplayName: POLESTR; out pchEaten: ULONG; out ppidl: PItemIDList;
	var dwAttributes: ULONG): HResult;
begin
	Result := E_NotImpl;
end;

function TSubFolder.SetNameOf(hwndOwner: HWND; pidl: PItemIDList;
	lpszName: POLEStr; uFlags: DWORD; var ppidlOut: PItemIDList): HResult;
begin
	Result := E_NotImpl;
end;

initialization
	TComObjectFactory.Create(ComServer, TSubFolder, Class_SubFolder,
		'SubFolder', '', ciInternal, tmApartment);
end.
