unit SubViewUnit;

interface

uses
	LinkUnit, SubFolderUnit,
	Windows, SysUtils, ActiveX, ComObj, ShlObj, CommCtrl;

type
	TSubView = class(TComObject, IOleWindow, IShellView)
	private
		FData: TSubFolderItem;
		FItems: TVarItems;
		FHandle: HWnd;
		FBrowser: IShellBrowser;
	protected
		function GetWindow(out wnd: HWnd): HResult; virtual; stdcall;
		function ContextSensitiveHelp(fEnterMode: BOOL): HResult; virtual; stdcall;
		function TranslateAccelerator(var Msg: TMsg): HResult; virtual; stdcall;
		function EnableModeless(Enable: Boolean): HResult; virtual; stdcall;
		function UIActivate(State: UINT): HResult; virtual; stdcall;
		function Refresh: HResult; virtual; stdcall;
		function CreateViewWindow(PrevView: IShellView;
			var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
			var Rect: TRect; out Wnd: HWND): HResult; virtual; stdcall;
		function DestroyViewWindow: HResult; virtual; stdcall;
		function GetCurrentInfo(out FolderSettings: TFolderSettings): HResult; virtual; stdcall;
		function AddPropertySheetPages(Reseved: DWORD;
			lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult; virtual; stdcall;
		function SaveViewState: HResult; virtual; stdcall;
		function SelectItem(pidl: PItemIDList; flags: UINT): HResult; virtual; stdcall;
		function GetItemObject(Item: UINT; const iid: TIID; IPtr: Pointer): HResult; virtual; stdcall;
		property Handle: HWND read FHandle;
	public
		constructor Create(const Data: TSubFolderItem);
		destructor Destroy; override;
	end;

const
	Class_SubView: TGUID = '{595D2825-10B5-11D5-A253-00900001489D}';

function GetVars(Window: HWnd; const Folder: TSubFolderItem; out Items: TVarItems): Boolean;
function AttrToStr(Attr: TCalcVarAttr; CalcType: TCalcType): string;
function TypeIDToStr(TypeID: TCalcVarTypeID; CalcType: TCalcType): string;

implementation

uses
	ComServ, ShellAPI;

function VarListCallBack(ID: Pointer; const NewVar: TCalcVar): TCalcBool; stdcall;
var
	Items: PVarItems;
begin
	Items := ID;
	SetLength (Items^, Length (Items^) + 1);
	Items^ [High (Items^)] := NewVar;
	Result := True;
end;

function GetVars(Window: HWnd; const Folder: TSubFolderItem; out Items: TVarItems): Boolean;
var
	Link: TLinkHandle;
begin
	SetLength (Items, 0);
	if TryOpenConnection (Link, Window) then begin
		Result := GetVarList (Link, PChar (AnsiString (Folder.Name)), VarListCallBack, @Items);
		if not Result then
			MessageBox (Window, PChar (UIStrings [UIVarListError]), PChar (UIStrings [UIError]), MB_OK or MB_ICONERROR);
		CloseConnection (Link);
	end else
		Result := False;
end;

function AttrToStr(Attr: TCalcVarAttr; CalcType: TCalcType): string;
begin
	Result := '';
	if (Attr and CALC_VAR_ATTR_ARCHIVED) <> 0 then
		Insert ('A', Result, Length (Result) + 1);
	if (Attr and CALC_VAR_ATTR_LOCKED) <> 0 then
		Insert ('R', Result, Length (Result) + 1);
end;

function TypeIDToStr(TypeID: TCalcVarTypeID; CalcType: TCalcType): string;
var
	StrID: Integer;
begin
	case TypeID of
		$00: StrID := UIVarExpr;
		$04: StrID := UIVarList;
		$06: StrID := UIVarMat;
		$0A: StrID := UIVarData;
		$0B: StrID := UIVarText;
		$0C: StrID := UIVarStr;
		$0D: StrID := UIVarGDB;
		$0E: StrID := UIVarFig;
		$10: StrID := UIVarPic;
		$12: StrID := UIVarPrgm;
		$13: StrID := UIVarFunc;
		$14: StrID := UIVarMac;
		$1C: StrID := UIVarOther;
		$1D: StrID := UIVarBackup;
		$1F: StrID := UIVarDir;
		$21: StrID := UIVarAsm;
		$22: StrID := UIVarIDList;
		$24: StrID := UIVarFlash;
		else StrID := UIVarUnknown;
	end;
	Result := UIStrings [StrID];
end;

{ TSubView }

function TSubView.AddPropertySheetPages(Reseved: DWORD;
	lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.ContextSensitiveHelp(fEnterMode: BOOL): HResult;
begin
	Result := E_NotImpl;
end;

constructor TSubView.Create(const Data: TSubFolderItem);
begin
	inherited Create;
	Inc (DLLRefCount);
	FData := Data;
end;

function TSubView.CreateViewWindow(PrevView: IShellView;
	var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
	var Rect: TRect; out Wnd: HWND): HResult;
var
	ParentWindow: HWND;
	ColData: TLVColumn;
	FileInfo: TSHFileInfo;
begin
	FBrowser := ShellBrowser;
	FBrowser.GetWindow (ParentWindow);
	FHandle := CreateWindowEx (WS_EX_CLIENTEDGE, WC_LISTVIEW, 'Calculator', WS_VISIBLE or WS_CHILD or WS_CLIPSIBLINGS or LVS_REPORT or LVS_SHAREIMAGELISTS, Rect.Left, Rect.top, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top, ParentWindow, 0, 0, nil);
	Wnd := Handle;
	ListView_SetImageList (Handle, SHGetFileInfo ('C:\', 0, FileInfo, SizeOf (FileInfo), SHGFI_SYSICONINDEX or SHGFI_SMALLICON), LVSIL_SMALL);
	ListView_SetImageList (Handle, SHGetFileInfo ('C:\', 0, FileInfo, SizeOf (FileInfo), SHGFI_SYSICONINDEX or SHGFI_LARGEICON), LVSIL_NORMAL);
	ColData.Mask := LVCF_FMT or LVCF_WIDTH or LVCF_TEXT;
	ColData.Fmt := LVCFMT_LEFT;
	ColData.cX := 150;
	ColData.pszText := PChar (UIStrings [UIVariableName]);
	ListView_InsertColumn (Handle, 0, ColData);
	ColData.pszText := PChar (UIStrings [UIVariableType]);
	ListView_InsertColumn (Handle, 1, ColData);
	ColData.Fmt := LVCFMT_RIGHT;
	ColData.cX := 50;
	ColData.pszText := PChar (UIStrings [UIVariableSize]);
	ListView_InsertColumn (Handle, 2, ColData);
	ColData.cX := 60;
	ColData.pszText := PChar (UIStrings [UIVariableAttr]);
	ListView_InsertColumn (Handle, 3, ColData);
	Result := Refresh;
end;

destructor TSubView.Destroy;
begin
	Dec (DLLRefCount);
	inherited;
end;

function TSubView.DestroyViewWindow: HResult;
begin
	if FHandle <> 0 then begin
		DestroyWindow (FHandle);
		FHandle := 0;
	end;
	Result := S_OK;
end;

function TSubView.EnableModeless(Enable: Boolean): HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.GetCurrentInfo(out FolderSettings: TFolderSettings):
	HResult;
begin
	with FolderSettings do begin
		ViewMode := FVM_DETAILS;
		fFlags := 0;
	end;
	Result := S_OK;
end;

function TSubView.GetItemObject(Item: UINT; const iid: TIID;
	IPtr: Pointer): HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.GetWindow(out wnd: HWnd): HResult;
begin
	Wnd := Handle;
	Result := S_OK;
end;

function TSubView.Refresh: HResult;
const
	Widths: array [0..1] of Integer = (100, -1);
var
	I: Integer;
	Item: TLVItem;
	S: string;
	Total: Cardinal;
begin
	ListView_DeleteAllItems (Handle);
	if GetVars (0, FData, FItems) then begin
		Total := 0;
		with Item do begin
			iItem := 0;
			iSubItem := 0;
			iImage := 0;
		end;
		for I := Low (FItems) to High (FItems) do begin
			Item.Mask := LVIF_TEXT or LVIF_IMAGE;
			Item.iSubItem := 0;
			Item.pszText := FItems[I].Name;
			ListView_InsertItem (Handle, Item);
			Item.Mask := LVIF_TEXT;
			Item.iSubItem := 1;
			Item.pszText := PChar (TypeIDToStr (FItems[I].TypeID, FItems[I].CalcType));
			ListView_SetItem (Handle, Item);
			Item.iSubItem := 2;
			Item.pszText := PChar (IntToStr (FItems[I].Size));
			ListView_SetItem (Handle, Item);
			Item.iSubItem := 3;
			Item.pszText := PChar (AttrToStr (FItems[I].Attr, FItems[I].CalcType));
			ListView_SetItem (Handle, Item);
			Inc (Item.iItem);
			Inc (Total, FItems[I].Size);
		end;
		FBrowser.SendControlMsg (FCW_STATUS, SB_SETPARTS, 2, Cardinal (@Widths), 0);
		if Length (FItems) = 1 then
			S := UIStrings [UIVarCount1]
		else
			S := Format (UIStrings [UIVarCount], [Length (FItems)]);
		FBrowser.SendControlMsg (FCW_STATUS, SB_SETTEXT, 0, Cardinal (PChar (S)), 0);
		FBrowser.SendControlMsg (FCW_STATUS, SB_SETTEXT, 1, Cardinal (PChar (Format (UIStrings [UITotalBytes], [Total]))), 0);
		Result := S_OK;
	end else
		Result := E_Fail;
end;

function TSubView.SaveViewState: HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.SelectItem(pidl: PItemIDList; flags: UINT): HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.TranslateAccelerator(var Msg: TMsg): HResult;
begin
	Result := E_NotImpl;
end;

function TSubView.UIActivate(State: UINT): HResult;
begin
	Result := S_OK;
end;

initialization
	TComObjectFactory.Create(ComServer, TSubView, Class_SubView,
		'SubView', '', ciInternal, tmApartment);
end.
