unit ObjectFileUnit;

interface

uses
	Classes;

type
	TCalcInt1 = packed record
		Int: Byte;
	end;
	PCalcInt1 = ^TCalcInt1;

	TCalcInt2 = packed record
		Hi,
		Lo: TCalcInt1;
	end;
	PCalcInt2 = ^TCalcInt2;

	TCalcInt4 = packed record
		Hi,
		Lo: TCalcInt2;
	end;
	PCalcInt4 = ^TCalcInt4;

	TPCInt1 = packed record
		Int: Byte;
	end;
	PPCInt1 = ^TPCInt1;

	TPCInt2 = packed record
		Lo,
		Hi: TPCInt1;
	end;
	PPCInt2 = ^TPCInt2;

	TPCInt4 = packed record
		Lo,
		Hi: TPCInt2;
	end;
	PPCInt4 = ^TPCInt4;

function ReadCalcInt(Int: TCalcInt1): Byte; overload;
function ReadCalcInt(Int: TCalcInt2): Word; overload;
function ReadCalcInt(Int: TCalcInt4): LongWord; overload;

procedure WriteCalcInt(Input: Byte; out Int: TCalcInt1); overload;
procedure WriteCalcInt(Input: Word; out Int: TCalcInt2); overload;
procedure WriteCalcInt(Input: LongWord; out Int: TCalcInt4); overload;

function ReadPCInt(Int: TPCInt1): Byte; overload;
function ReadPCInt(Int: TPCInt2): Word; overload;
function ReadPCInt(Int: TPCInt4): LongWord; overload;

procedure WritePCInt(Input: Byte; out Int: TPCInt1); overload;
procedure WritePCInt(Input: Word; out Int: TPCInt2); overload;
procedure WritePCInt(Input: LongWord; out Int: TPCInt4); overload;

type
	TCOFFFileHeader = packed record
		Machine: TCalcInt2;
		SectionCount: TCalcInt2;
		TimeDateStamp: TCalcInt4;
		SymbolTableAddress: TCalcInt4;
		SymbolCount: TCalcInt4;
		OptionalHeaderSize: TCalcInt2;
		Flags: TCalcInt2;
	end;

	TCOFFSectionHeader = packed record
		SectionName: array [0..7] of Char;
		VirtualSize: TCalcInt4;
		VirtualAddress: TCalcInt4;
		Size: TCalcInt4;
		Address: TCalcInt4;
		RelocationAddress: TCalcInt4;
		LineAddress: TCalcInt4;
		RelocationCount: TCalcInt2;
		LineCount: TCalcInt2;
		Flags: TCalcInt4;
	end;

	TCOFFSymbol = packed record
		SymbolName: array [0..7] of Char;
		Value: TCalcInt4;
		SectionNumber: TCalcInt2;
		SymbolType: TCalcInt2;
		StorageClass: TCalcInt1;
		AuxSymbolCount: TCalcInt1;
	end;

	TCOFFSymbolFileName = array [0..(SizeOf(TCOFFSymbol)-1)] of Char;

	TCOFFLine = packed record
		LineAddress: TCalcInt4;
		LineNumber: TCalcInt2;
	end;

procedure ReadCOFFFileHeader(Stream: TStream; out Header: TCOFFFileHeader);
procedure ReadCOFFSectionHeader(Stream: TStream; out Header: TCOFFSectionHeader);
procedure ReadCOFFSymbol(Stream: TStream; out Symbol: TCOFFSymbol);
procedure ReadCOFFLine(Stream: TStream; out Line: TCOFFLine);

type
	TCOFFHeaders = record
		FileHeader: TCOFFFileHeader;
		SectionHeaders: array of TCOFFSectionHeader;
	end;

	TCOFFLines = array of TCOFFLine;

	TCOFFSymbols = array of TCOFFSymbol;

	TCOFFFileLine = record
		LineAddress: Integer;
		LineNumber: Integer;
	end;

	TCOFFFileLines = record
		FileName: string;
		Lines: array of TCOFFFileLine;
	end;

procedure ReadCOFFHeaders(Stream: TStream; out Headers: TCOFFHeaders);
procedure ReadCOFFSymbols(Stream: TStream; const Header: TCOFFFileHeader; out Symbols: TCOFFSymbols);
procedure ReadCOFFLines(Stream: TStream; const Section: TCOFFSectionHeader; out Lines: TCOFFLines);
procedure ReadCOFFStrings(Stream: TStream; const Header: TCOFFFileHeader; OutStream: TMemoryStream);

procedure ExtractCOFFFileLines(const Section: TCOFFSectionHeader; const FileName: string; const Symbols: TCOFFSymbols; const Strings: TStream; const Lines: TCOFFLines; out FileLines: TCOFFFileLines);
procedure GetCOFFFileNameSymbolIndex(const Symbols: TCOFFSymbols; const Strings: TStream; const FileName: string; out FirstSymbol, LastSymbol: Integer);

implementation

uses
	SysUtils;

{ Functions to read integers machine-independently }

function ReadCalcInt(Int: TCalcInt1): Byte;
begin
	Result := Int.Int;
end;

function ReadCalcInt(Int: TCalcInt2): Word;
begin
	Result := ReadCalcInt (Int.Hi) shl (SizeOf (Int.Lo) * 8) or ReadCalcInt (Int.Lo);
end;

function ReadCalcInt(Int: TCalcInt4): LongWord;
begin
	Result := ReadCalcInt (Int.Hi) shl (SizeOf (Int.Lo) * 8) or ReadCalcInt (Int.Lo);
end;

function ReadPCInt(Int: TPCInt1): Byte;
begin
	Result := Int.Int;
end;

function ReadPCInt(Int: TPCInt2): Word;
begin
	Result := ReadPCInt (Int.Hi) shl (SizeOf (Int.Lo) * 8) or ReadPCInt (Int.Lo);
end;

function ReadPCInt(Int: TPCInt4): LongWord;
begin
	Result := ReadPCInt (Int.Hi) shl (SizeOf (Int.Lo) * 8) or ReadPCInt (Int.Lo);
end;

{ Functions to write integers machine-independently }

procedure WriteCalcInt(Input: Byte; out Int: TCalcInt1);
begin
	Int.Int := Input;
end;

procedure WriteCalcInt(Input: Word; out Int: TCalcInt2);
begin
	WriteCalcInt (Byte (Input shr (SizeOf (Int.Lo) * 8)), Int.Hi);
	WriteCalcInt (Byte (Input), Int.Lo);
end;

procedure WriteCalcInt(Input: LongWord; out Int: TCalcInt4);
begin
	WriteCalcInt (Word (Input shr (SizeOf (Int.Lo) * 8)), Int.Hi);
	WriteCalcInt (Word (Input), Int.Lo);
end;

procedure WritePCInt(Input: Byte; out Int: TPCInt1);
begin
	Int.Int := Input;
end;

procedure WritePCInt(Input: Word; out Int: TPCInt2);
begin
	WritePCInt (Byte (Input shr (SizeOf (Int.Lo) * 8)), Int.Hi);
	WritePCInt (Byte (Input), Int.Lo);
end;

procedure WritePCInt(Input: LongWord; out Int: TPCInt4);
begin
	WritePCInt (Word (Input shr (SizeOf (Int.Lo) * 8)), Int.Hi);
	WritePCInt (Word (Input), Int.Lo);
end;

{ Functions to directly read data }

procedure ReadCOFFFileHeader(Stream: TStream; out Header: TCOFFFileHeader);
begin
	Stream.ReadBuffer (Header, SizeOf (Header));
end;

procedure ReadCOFFSectionHeader(Stream: TStream; out Header: TCOFFSectionHeader);
begin
	Stream.ReadBuffer (Header, SizeOf (Header));
end;

procedure ReadCOFFSymbol(Stream: TStream; out Symbol: TCOFFSymbol);
begin
	Stream.ReadBuffer (Symbol, SizeOf (Symbol));
end;

procedure ReadCOFFLine(Stream: TStream; out Line: TCOFFLine);
begin
	Stream.ReadBuffer (Line, SizeOf (Line));
end;

{ Functions to read data into arrays }

procedure ReadCOFFHeaders(Stream: TStream; out Headers: TCOFFHeaders);
var
	I: Integer;
begin
	Stream.Position := 0;
	with Headers do begin
		ReadCOFFFileHeader (Stream, FileHeader);
		Stream.Seek (ReadCalcInt (FileHeader.OptionalHeaderSize), soFromCurrent);
		SetLength (SectionHeaders, ReadCalcInt (FileHeader.SectionCount));
		for I := Low (SectionHeaders) to High (SectionHeaders) do
			ReadCOFFSectionHeader (Stream, SectionHeaders [I]);
	end;
end;

procedure ReadCOFFSymbols(Stream: TStream; const Header: TCOFFFileHeader; out Symbols: TCOFFSymbols);
var
	I: Integer;
begin
	Stream.Position := ReadCalcInt (Header.SymbolTableAddress);
	SetLength (Symbols, ReadCalcInt (Header.SymbolCount));
	for I := Low (Symbols) to High (Symbols) do
		ReadCOFFSymbol (Stream, Symbols [I]);
end;

procedure ReadCOFFLines(Stream: TStream; const Section: TCOFFSectionHeader; out Lines: TCOFFLines);
var
	I: Integer;
begin
	Stream.Position := ReadCalcInt (Section.LineAddress);
	SetLength (Lines, ReadCalcInt (Section.LineCount));
	for I := Low (Lines) to High (Lines) do
		ReadCOFFLine (Stream, Lines [I]);
end;

procedure ReadCOFFStrings(Stream: TStream; const Header: TCOFFFileHeader; OutStream: TMemoryStream);
var
	Pos: Integer;
	Size: TCalcInt4;
begin
	Pos := ReadCalcInt (Header.SymbolTableAddress) + ReadCalcInt (Header.SymbolCount) * SizeOf (TCOFFSymbol);
	Stream.Position := Pos;
	Stream.ReadBuffer (Size, SizeOf (Size));
	Stream.Position := Pos;
	OutStream.CopyFrom (Stream, ReadCalcInt (Size));
end;

procedure ExtractCOFFFileLines(const Section: TCOFFSectionHeader; const FileName: string; const Symbols: TCOFFSymbols; const Strings: TStream; const Lines: TCOFFLines; out FileLines: TCOFFFileLines);
var
	I,
	FirstSymbol,
	LastSymbol,
	CurSymbol: Integer;
begin
	FileLines.FileName := FileName;
	SetLength (FileLines.Lines, 0);
	GetCOFFFileNameSymbolIndex (Symbols, Strings, FileName, FirstSymbol, LastSymbol);
	if FirstSymbol >= 0 then begin
		CurSymbol := 0;
		for I := Low (Lines) to High (Lines) do
			if ReadCalcInt (Lines[I].LineNumber) > 0 then begin
				if (CurSymbol >= FirstSymbol) and (CurSymbol <= LastSymbol) then begin
					SetLength (FileLines.Lines, Length (FileLines.Lines) + 1);
					with FileLines.Lines [High (FileLines.Lines)] do begin
						LineAddress := ReadCalcInt (Lines[I].LineAddress);
						LineNumber := ReadCalcInt (Lines[I].LineNumber);
					end;
				end;
			end else
				CurSymbol := ReadCalcInt (Lines[I].LineAddress);
	end;
end;

procedure GetCOFFFileNameSymbolIndex(const Symbols: TCOFFSymbols; const Strings: TStream; const FileName: string; out FirstSymbol, LastSymbol: Integer);
type
	PLongWord = ^LongWord;
var
	I,
	P: Integer;
	C: Char;
begin
	FirstSymbol := -1;
	LastSymbol := High (Symbols);
	for I := Low (Symbols) to High (Symbols) do
		if Symbols[I].SymbolName = '.file' then begin
			if FirstSymbol >= 0 then begin
				LastSymbol := I - 1;
				Break;
			end else if I + 1 <= High (Symbols) then begin
				if (PLongWord (@(Symbols[I+1].SymbolName[0])))^ = 0 then begin
					Strings.Position := ReadCalcInt ((PCalcInt4 (@(Symbols[I+1].SymbolName[4])))^);
					C := #0;
					for P := 1 to Length (FileName) do begin
						Strings.ReadBuffer (C, 1);
						if UpCase (C) <> UpCase (FileName [P]) then begin
							C := #0;
							Break;
						end;
					end;
					if C <> #0 then begin
						Strings.ReadBuffer (C, 1);
						if C = #0 then
							FirstSymbol := I;
					end;
				end else if UpperCase (TCOFFSymbolFileName (Symbols [I + 1])) = UpperCase (FileName) then
					FirstSymbol := I;
			end;
		end;
	if FirstSymbol < 0 then
		LastSymbol := -1;
end;

end.

