unit TemplateWizardUnit;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls;

const
	CFileStart = 0;
	CFileEnd = 4;
	HdrFileStart = 5;
	HdrFileEnd = 5;

	CalcSelPage = 0;
	ReturnToVarPage = 3;
	CommentPage = 4;

type
	TTemplateWizardForm = class(TForm)
		Panel1: TPanel;
    WizardImage: TImage;
		Bevel1: TBevel;
		BackButton: TButton;
		NextButton: TButton;
		CancelButton: TButton;
		PageController: TNotebook;
		Label1: TLabel;
		Label2: TLabel;
		TI89CheckBox: TCheckBox;
		TI92PlusCheckBox: TCheckBox;
		V200CheckBox: TCheckBox;
		Label3: TLabel;
		NoStubRadioButton: TRadioButton;
		DoorsRadioButton: TRadioButton;
		Label4: TLabel;
		Label5: TLabel;
		Label6: TLabel;
		CommentCheckBox: TCheckBox;
		CommentEditBox: TEdit;
		Label7: TLabel;
		LCDSaveCheckBox: TCheckBox;
		Label10: TLabel;
		ReturnNoneRadioButton: TRadioButton;
		ReturnDoneRadioButton: TRadioButton;
		ReturnValueRadioButton: TRadioButton;
		Label11: TLabel;
		Label12: TLabel;
		Label13: TLabel;
		CHdrFileRadioButton: TRadioButton;
		GNUAsmHdrFileRadioButton: TRadioButton;
		A68kAsmHdrFileRadioButton: TRadioButton;
		Label14: TLabel;
		Label16: TLabel;
		Label17: TLabel;
		ReturnToVarCheckBox: TCheckBox;
		ReturnVariableEdit: TEdit;
		OptimizeROMCallsCheckBox: TCheckBox;
    MinAMSCheckBox: TCheckBox;
    MinAMSEdit: TEdit;
    EnableErrorReturnCheckBox: TCheckBox;
		procedure FormShow(Sender: TObject);
		procedure NextButtonClick(Sender: TObject);
		procedure BackButtonClick(Sender: TObject);
		procedure CalcCheckBoxClick(Sender: TObject);
		procedure CommentCheckBoxClick(Sender: TObject);
		procedure PageControllerPageChanged(Sender: TObject);
		procedure ShellRadioButtonClick(Sender: TObject);
		procedure ReturnToVarCheckBoxClick(Sender: TObject);
    procedure ReturnVariableEditChange(Sender: TObject);
    procedure ReturnValueCheckBoxClick(Sender: TObject);
    procedure MinAMSCheckBoxClick(Sender: TObject);
	private
	public
		FirstPage: Integer;
		LastPage: Integer;
		function GetNextPage(FlipDir: Integer): Integer;
	end;

implementation

{$R *.DFM}

function TTemplateWizardForm.GetNextPage(FlipDir: Integer): Integer;
var
	FlipAgain: Boolean;
begin
	Result := PageController.PageIndex;
	repeat
		FlipAgain := False;
		Inc (Result, FlipDir);
		if (Result < FirstPage) or (Result > LastPage) then begin
			Result := -1;
			Exit;
		end;
		if
			 ((Result = ReturnToVarPage) and
				(not ReturnValueRadioButton.Checked))
			or
			 ((Result = CommentPage) and
				(not DoorsRadioButton.Checked))
			then
				FlipAgain := True;
	until not FlipAgain;
end;

procedure TTemplateWizardForm.FormShow(Sender: TObject);
begin
	PageController.PageIndex := FirstPage;
	ActiveControl := PageController;
end;

procedure TTemplateWizardForm.NextButtonClick(Sender: TObject);
var
	NewPage: Integer;
begin
	NewPage := GetNextPage (1);
	if NewPage < 0 then
		ModalResult := mrOK
	else
		PageController.PageIndex := NewPage;
end;

procedure TTemplateWizardForm.BackButtonClick(Sender: TObject);
var
	NewPage: Integer;
begin
	NewPage := GetNextPage (-1);
	if NewPage >= 0 then
		PageController.PageIndex := NewPage;
end;

procedure TTemplateWizardForm.CalcCheckBoxClick(Sender: TObject);
begin
	if PageController.PageIndex = CalcSelPage then
		NextButton.Enabled := TI89CheckBox.Checked or TI92PlusCheckBox.Checked;
end;

procedure TTemplateWizardForm.CommentCheckBoxClick(Sender: TObject);
begin
	CommentEditBox.Enabled := CommentCheckBox.Checked;
	if not CommentCheckBox.Checked then
		CommentEditBox.Text := '';
end;

procedure TTemplateWizardForm.PageControllerPageChanged(Sender: TObject);
begin
	with PageController do begin
		BackButton.Enabled := GetNextPage (-1) >= 0;
		if GetNextPage (1) < 0 then
			NextButton.Caption := '&Finish'
		else
			NextButton.Caption := '&Next >';
		CalcCheckBoxClick (Sender);
		ReturnVariableEditChange (Sender);
	end;
end;

procedure TTemplateWizardForm.ShellRadioButtonClick(Sender: TObject);
begin
	with LCDSaveCheckBox do begin
		Enabled := not DoorsRadioButton.Checked;
		if not Enabled then
			Checked := True;
	end;
	with OptimizeROMCallsCheckBox do begin
		Enabled := not DoorsRadioButton.Checked;
		if not Enabled then
			Checked := False;
	end;
end;

procedure TTemplateWizardForm.ReturnToVarCheckBoxClick(Sender: TObject);
begin
	ReturnVariableEdit.Enabled := ReturnToVarCheckBox.Checked;
	ReturnVariableEditChange (Sender);
end;

procedure TTemplateWizardForm.ReturnVariableEditChange(Sender: TObject);
var
	E: Boolean;
	I: Integer;
begin
	if PageController.PageIndex = ReturnToVarPage then begin
		if ReturnToVarCheckBox.Checked then
			with ReturnVariableEdit do begin
				E := (Length (Text) > 0) and (Length (Text) <= 8);
				if E then
					for I := 1 to Length (Text) do
						if not (Text [I] in ['a'..'z', '0'..'9']) then begin
							E := False;
							Break;
						end;
				NextButton.Enabled := E;
			end
		else
			NextButton.Enabled := True;
	end;
end;

procedure TTemplateWizardForm.ReturnValueCheckBoxClick(Sender: TObject);
begin
	PageControllerPageChanged (Sender);
end;

procedure TTemplateWizardForm.MinAMSCheckBoxClick(Sender: TObject);
begin
	MinAMSEdit.Enabled := MinAMSCheckBox.Checked;
end;

end.
