/* 
   SHARED - some functions shared between tprbuilder, tigcc and some others...
  
   Copyright (C) 2002 Romain Livin

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/


#ifndef _MISC_C
#define _MISC_C

#include <stdio.h>
#include "misc.h"

/* 
   Determine the size of a file array 
*/
int dyn_file_array_size(char **array, int *size)
{
    char **p;
    
    *size = 0;
    if(array != NULL) {
    	for(p=array; *p != NULL; p++, (*size)++);
    }
    
    return *size;
}

/* 
   Add a filename to a filename array while maintaing a file count 
*/
int dyn_file_array_add(char *file, char ***array, int *file_count) 
{
    int len = strlen(file) + 1;
    int nfiles = 0;
    char **p;
    char **files = *array;
    int i;
    
    // Determine size of array
    dyn_file_array_size(files, &nfiles);
    
    // Resize array
    files = (char **) realloc(files, (nfiles + 2) * sizeof(char *));
    if (files == NULL) {
	fprintf(stderr, "Fatal error: not enough free memory\n");
	exit(-1);
    }
    
    // Allocate and copy string
    if ((files[nfiles] = (char *)calloc(len, sizeof(char))) == NULL) {
	fprintf(stderr, "Fatal error: not enough free memory\n");
	exit(-1);
    }
    
    // Maintain array consistency
    strcpy(files[nfiles], file);
    files[++nfiles] = NULL;
    *file_count = nfiles;
    *array = files;
    
    return TRUE;
}

/* 
   Free a file array and the associated strings 
*/
void dyn_file_array_free(char **files) 
{
    int i, count;
    
    if(files == NULL) return;
    dyn_file_array_size(files, &count);
    
    for(i=0; i<count; i++) {
    	free(files[i]);
    }
    
    free(files);
}

/* 
   Encapsulate long filenames with "".
   The returned string must be freed when no longer used
*/
char* encapsulate_long_filename(const char *file)
{
    char *s;
    
    if ((s = calloc(strlen(file)+1+2, sizeof(char))) == NULL) {
	fprintf(stderr, "Fatal error: not enough free memory\n");
	exit(-1);
    }
    
    if(!strchr(file, ' '))
	strcpy(s, file);
    else {
	strcpy(s, "\"");
	strcat(s, file);
	strcat(s, "\"");
    }
    
    return s;
}

/*
  Remove beginning & ending '"' from files.
  The string is modified in place !
*/
char* tail_long_filename(char *file)
{
    char *p = file;

    if( (file[0] != '\"') && (file[strlen(file)] != '\"') )
	return file;
    
    strcpy(file, ++p);     /* remove first quote */
    p[strlen(p)-1] = '\0'; /* remove last quote */

    return file;
}

/* 
   Duplicate a string by allocating a new buffer 
*/
/*
char *strdup(const char *s)
{
    char *d = (char *)malloc((strlen(s) + 1) * sizeof(char));
    if(d == NULL)
	return NULL;
    else
	return strcpy(d, s);
}
*/

/* 
   Change filename extension and keep long filenames.
   If the filename has no extension, then the extension is append
   Dynamic allocation to do ...
*/
char *change_extension(char *file, char *newext) 
{
    char *start = (char *)strrchr(file, '.');
    if(start == NULL) {
	start = file; 	//return file;
    } else {
	sprintf(start, "%s", newext);
    }    

    if(file[0] == '\"') strcat(start, "\"");
    
    return file;
}

/* 
   Find a token in a string and return the subsequent string 
*/
char *find_str(char *s, const char *t)
{
  char *p=(char *)strstr(s, t);
  
  if(p==NULL) return NULL;
  else return p+strlen(t);
}

#endif
