/*  TPRbuilder - A make program for TIGCC project files (tpr).
 *
 *  Copyright (C) 2002  Romain Lievin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributelabeld in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TMAKE_H
#define _TMAKE_H

/* GNU stuffs */
enum {DUMMY_CODE=129};
#define EXIT_FAILURE 1

/* global enumerations */
enum SectionType_ { SECTION_NONE, SECTION_SETTINGS, SECTION_FILES };
typedef enum SectionType_ SectionType;

/* constants */
#define MAX_CHARS	65535
#define MAX_ARRAY_FILES 256

#define BIN_BASE		"m68k-coff-"
#define DEVNULL                 "> /dev/null"

#define TOKEN_SETTINGS 	 "[Settings]"
#define TOKEN_ARCHIVE    "Archive="
#define TOKEN_PACK       "Pack="
#define TOKEN_PACKNAME   "Packed Variable="
#define TOKEN_PRJNAME    "Project Name="
#define TOKEN_CC	 "GCC Switches="
#define TOKEN_A68K 	 "Assembler Switches="
#define TOKEN_CV     	 "Linker Switches="
#define TOKEN_LD         "GNU Linker Switches="
#define TOKEN_AS	 "GNU Assembler Switches="
#define TOKEN_BSRPATCH   "BSR Patch="
#define TOKEN_DEBUGINFO  "Debug Info="
#define TOKEN_STDLIB	 "Standard Library="
#define TOKEN_CMDLINE	 "Command Line="
#define TOKEN_POSTBUILD  "Post-Build Process="

#define TOKEN_FILES	 "[Included Files]"
#define TOKEN_CFILE	 "C File %i="
#define TOKEN_SFILE	 "GNU Assembler File %i="
#define TOKEN_HFILE	 "Header File %i="
#define TOKEN_ASMFILE    "Assembler File %i="
#define TOKEN_AFILE      "Archive File %i="
#define TOKEN_TFILE      "Text File %i="
#define TOKEN_QLLFILE    "Quill File %i="

/* definitions */
typedef struct _settings Settings;
struct _settings
{
    Boolean 	archive;         // we want to build an archive
    Boolean 	quill;           // we want to build a Quill project
    Boolean 	pack;            // we want to pack the executable
    Boolean 	bsr_patch;       // use BSR patches
    Boolean 	debug_info;      // pass -g option to tigcc
    Boolean 	std_lib;         // link against tigcc.a

    char 	*pack_name;
    char 	*prj_name;
    char 	*cc_switches;
    char 	*as_switches;
    char 	*ld_switches;
    char 	*cv_switches;
    char 	*a68k_switches;
    char 	*post_build;
    char 	*cmdline;
};

/* Local declarations */
static int decode_switches (int argc, char **argv);
static void usage (int status);
static void safe_exit(void);

int  file_array_add(char *file, char ***files, int *file_count);
void file_array_free(char **files);
int  file_array_size(char **files, int *count);

char* encapsulate_long_filename(const char *file);
char* strip(char *str);
void  stop (int line);
char *find_str(char *s, const char *t);
char *find_param_str(char *s, const char*t, int *i);
//char *strdup(const char *s);

int process_settings(char *args);
int parse_file(const char *filename);

#endif
